from mimesis.providers import BaseProvider


class ClothingSizes(BaseProvider):
    """Class for generate clothing sizes data"""

    def international(self):
        """Get a random size in international format.

        :return: Clothing size.
        :Example:
            XXL.
        """
        sizes = (
            'L',
            'M',
            'S',
            'XL',
            'XS',
            'XXL',
            'XXS',
            'XXXL',
        )

        return self.random.choice(sizes)

    def european(self):
        """Generate a random clothing size in European format.

        :return: Clothing size.
        :Example:
            42
        """
        size = self.random.choice([i for i in range(40, 62) if i % 2 == 0])
        return size

    def custom(self, minimum=40, maximum=62):
        """Generate clothing size using custom format.

        :param minimum: Min value.
        :param maximum: Max value
        :return: Clothing size.
        :Example:
            44
        """
        return self.random.randint(int(minimum), int(maximum))



from binascii import hexlify
import hashlib
import uuid
import os

from mimesis.exceptions import UnsupportedAlgorithm
from mimesis.providers import BaseProvider


class Cryptographic(BaseProvider):
    """This class provides support cryptographic data.
    """

    def uuid(self):
        """Generate random UUID.

        :return: UUID
        """
        return str(uuid.UUID(int=self.random.getrandbits(128)))

    def hash(self, algorithm='sha1'):
        """Generate random hash.

        :param algorithm: Hashing algorithm ('md5', 'sha1', 'sha224', 'sha256',
        'sha384', 'sha512').
        :return: Hash.
        """
        algorithm = algorithm.lower().strip()

        if algorithm in hashlib.algorithms_guaranteed:
            if hasattr(hashlib, algorithm):
                fn = getattr(hashlib, algorithm)
                _hash = fn(self.uuid().encode())
                return _hash.hexdigest()
        else:
            raise UnsupportedAlgorithm(
                'Algorithm {0} is does not support. Use: {1}'.format(
                    algorithm, ', '.join(hashlib.algorithms_guaranteed)),
            )

    def bytes(self, entropy=None):
        """Get a random byte string containing *entropy* bytes.

        The string has *entropy* random bytes, each byte converted to two
        hex digits.

        :param entropy:
        :return: Bytes.
        """
        if entropy is None:
            entropy = 32

        return os.urandom(entropy)

    def token(self, entropy=None):
        """Return a random text string, in hexadecimal.

        :param entropy: Number of bytes.
        :return: Token.
        """
        return hexlify(self.bytes(entropy)).decode('ascii')



from random import Random

from mimesis.utils import locale_info


class BaseProvider(object):
    """This is a base class for all providers."""

    def __init__(self, locale='en', seed=None):
        """Base constructor for all providers.

        :param locale: Current locale. Default is 'en'.
        :param seed: Seed to all the random functions. Default is 'None'.
        """
        self.locale = locale
        self.seed = seed
        self.random = Random()

        if seed is not None:
            self.random.seed(self.seed)

    def __str__(self):
        """
        Nice and pretty representation of a class.
        """
        return '{}:{}:{}'.format(
            self.__class__.__name__,
            self.locale,
            locale_info(self.locale),
        )



from random import choice, randint

from mimesis.providers.code import Code

# Internal
_custom_code = Code().custom_code


class USASpecProvider(object):
    """Class that provides special data for en"""

    class Meta:
        name = 'usa_provider'

    @staticmethod
    def tracking_number(service='usps'):
        """Generate random tracking number for USPS, FedEx and UPS.
        :param service:
        :return:
        """
        service = service.lower()

        if service not in ('usps', 'fedex', 'ups'):
            raise ValueError('Unsupported post service')

        services = {
            'usps': (
                '#### #### #### #### ####',
                '@@ ### ### ### US',
            ),
            'fedex': (
                '#### #### ####',
                '#### #### #### ###',
            ),
            'ups': (
                '1Z@####@##########',
            ),
        }
        mask = choice(services[service])
        return _custom_code(mask=mask)

    @staticmethod
    def ssn():
        """Generate a random, but valid Social Security Number.

        :returns: Random SSN
        :Example:
            569-66-5801
        """
        # Valid SSNs exclude 000, 666, and 900-999 in the area group
        area = randint(1, 899)
        if area == 666:
            area = 665

        return '{:03}-{:02}-{:04}'.format(
            area, randint(1, 99), randint(1, 9999))

    @staticmethod
    def personality(category='mbti'):
        """Generate a type of personality.

        :param category: Category.
        :return: Personality type.
        :Example:
            ISFJ.
        """
        mbtis = ('ISFJ', 'ISTJ', 'INFJ', 'INTJ',
                 'ISTP', 'ISFP', 'INFP', 'INTP',
                 'ESTP', 'ESFP', 'ENFP', 'ENTP',
                 'ESTJ', 'ESFJ', 'ENFJ', 'ENTJ')

        if category.lower() == 'rheti':
            return randint(1, 10)

        return choice(mbtis)



from mimesis.data import CURRENCIES, CURRENCY_SYMBOLS
from mimesis.providers import BaseProvider
from mimesis.utils import pull


class Business(BaseProvider):
    """Class for generating data for business."""

    def __init__(self, *args, **kwargs):
        """
        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self.data = pull('business.json', self.locale)

    def company_type(self, abbr=False):
        """Get a random type of business entity.

        :param abbr: If True then return abbreviated company type.
        :return: Types of business entity.
        :Example:
            Incorporated.
        """
        key = 'abbr' if abbr else 'title'
        company_type = self.data['company']['type'][key]
        return self.random.choice(company_type)

    def company(self):
        """Get a random company name.

        :return: Company name.
        :Example:
            Gamma Systems.
        """
        companies = self.data['company']['name']
        return self.random.choice(companies)

    def copyright(self):
        """Generate a random copyright.

        :return: Dummy copyright of company.
        :Example:
            © 1990-2016 Komercia, Inc.
        """
        company_type = self.company_type(abbr=True)
        return '© %s, %s' % (self.company(), company_type)

    def currency_iso(self):
        """Get a currency code. ISO 4217 format.

        :return: Currency code.
        :Example:
            RUR.
        """
        return self.random.choice(CURRENCIES)

    def price(self, minimum=10.00, maximum=1000.00):
        """Generate a random price.

        :param minimum:
        :param maximum:
        :return: Price.
        :Example:
            599.99 $.
        """
        currencies = CURRENCY_SYMBOLS

        price = self.random.uniform(float(minimum), float(maximum))

        fmt = '{0:.2f} {1}'

        if self.locale in currencies:
            return fmt.format(price, currencies[self.locale])

        return fmt.format(price, currencies['default'])



import json

from mimesis.data import (CSS_PROPERTIES, CSS_SELECTORS, CSS_SIZE_UNITS,
                          HTML_CONTAINER_TAGS, HTML_MARKUP_TAGS)
from mimesis.providers import BaseProvider, Internet
from mimesis.providers.text import Text


class Structured(BaseProvider):
    """Provider for structured text data such as CSS, HTML, JSON etc."""

    def __init__(self, *args, **kwargs):
        """
        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self.internet = Internet()
        self.text = Text()

    def css(self):
        """Generates a random snippet of CSS.

        :return: CSS.
        :rtype: str
        """
        selector = self.random.choice(CSS_SELECTORS)
        css_sel = '{}{}'.format(selector, self.text.word())

        cont_tag = self.random.choice(list(HTML_CONTAINER_TAGS.keys()))
        mrk_tag = self.random.choice(HTML_MARKUP_TAGS)

        base = '{}'.format(self.random.choice([cont_tag, mrk_tag, css_sel]))
        props = '; '.join(
            [self.css_property() for _ in range(self.random.randint(1, 6))])
        return '{} {{{}}}'.format(base, props)

    def css_property(self):
        """Generates a random snippet of CSS that assigns value to a property.

        :return: CSS property.
        :rtype: str
        :Examples:
            'background-color: #f4d3a1'
        """
        prop = self.random.choice(list(CSS_PROPERTIES.keys()))
        val = CSS_PROPERTIES[prop]

        if isinstance(val, list):
            val = self.random.choice(val)
        elif val == 'color':
            val = self.text.hex_color()
        elif val == 'size':
            val = '{}{}'.format(self.random.randint(1, 99),
                                self.random.choice(CSS_SIZE_UNITS))

        return '{}: {}'.format(prop, val)

    def html(self):
        """Generate a random HTML tag with text inside and some attrs set.

        :return: HTML.
        :rtype: str
        :Examples:
            '<span class="select" id="careers">
                Ports are created with the built-in function open_port.
            </span>'
        """
        tag_name = self.random.choice(list(HTML_CONTAINER_TAGS))
        tag_attributes = list(HTML_CONTAINER_TAGS[tag_name])
        k = self.random.randint(1, len(tag_attributes))

        selected_attrs = self.random.sample(tag_attributes, k=k)

        attrs = []
        for attr in selected_attrs:
            attrs.append('{}="{}"'.format(
                attr, self.html_attribute_value(tag_name, attr)))

        html_result = '<{tag} {attrs}>{content}</{tag}>'
        return html_result.format(
            tag=tag_name,
            attrs=' '.join(attrs),
            content=self.text.sentence(),
        )

    def html_attribute_value(self, tag, attribute):
        """Random value for specified HTML tag attribute.

        :param tag: An HTML tag.
        :param attribute: An attribute of the specified tag.
        :type tag: str
        :type attribute: str
        :return: An attribute.
        :rtype: str
        """
        try:
            value = HTML_CONTAINER_TAGS[tag][attribute]
        except KeyError:
            raise NotImplementedError(
                'Tag {} or attribute {} is not supported'.format(
                    tag, attribute))

        if isinstance(value, list):
            value = self.random.choice(value)
        elif value == 'css':
            value = self.css_property()
        elif value == 'word':
            value = self.text.word()
        elif value == 'url':
            value = self.internet.home_page()
        else:
            raise NotImplementedError(
                'Attribute type {} is not implemented'.format(value))
        return value

    def json(self, items=5, max_depth=3, recursive=False):
        """Generate a random snippet of JSON.

        :param items: Number of top-level items to produce.
        :param max_depth: Maximum depth of each top-level item.
        :param recursive: When used recursively, will return a Python object
        instead of JSON string.
        :return: JSON
        """

        # choose root element type
        root = self.random.choice([list, dict])()

        for _ in range(items):

            key = self.text.word()

            if max_depth > 0:
                value = self.random.choice([
                    self.text.sentence(),
                    self.random.randint(1, 10000),
                    self.random.random(),
                    self.json(max_depth=max_depth - 1,
                              recursive=True),
                ])
                if isinstance(root, list):
                    root.append(value)
                elif isinstance(root, dict):
                    root[key] = value

        if recursive:
            return root

        return json.dumps(root, indent=4)



import array

from .base import BaseProvider


class Numbers(BaseProvider):
    """Class for generating numbers"""

    def floats(self, n=2, type_code='f', to_list=False):
        """Generate an array of random float number of 10**n.

        +-----------+----------------+--------------+----------------------+
        | Type Code | C Type         | Storage size | Value range          |
        +===========+================+==============+======================+
        | 'f'       | floating point | 4 byte       | 1.2E-38 to 3.4E+38   |
        +-----------+----------------+--------------+----------------------+
        | 'd'       | floating point | 8 byte       | 2.3E-308 to 1.7E+308 |
        +-----------+----------------+--------------+----------------------+

        :param n: Raise 10 to the 'n' power.
        :param type_code: A code of type.
        :param to_list: Convert array to list.

        .. note:: When you work with large numbers, it is better not to use
            this option, because type 'array' much faster than 'list'.

        :return: An array of floating-point numbers.
        """
        nums = array.array(type_code, (self.random.random()
                                       for _ in range(10 ** int(n))))
        return nums.tolist() if to_list else nums

    @staticmethod
    def primes(start=1, end=999, to_list=False):
        """Generate an array of prime numbers of 10 ** n.

        +------------+-----------------+--------------+--------------------+
        | Type Code | C Type           | Storage size | Value range        |
        +===========+==================+==============+====================+
        | 'L'       | unsigned integer | 4 byte       | 0 to 4,294,967,295 |
        +-----------+------------------+--------------+--------------------+

        :param start: First value of range.
        :param end: Last value of range.
        :param to_list: Convert array to list.
        :return: An array of floating-point numbers.
        """
        nums = array.array('L', (i for i in range(start, end) if i % 2))
        return nums.tolist() if to_list else nums

    def digit(self, to_bin=False):
        """Get a random digit.

        :param to_bin: If True then convert to binary.
        :return: Digit.
        :Example:
            4.
        """
        digit = self.random.randint(0, 9)

        if to_bin:
            return bin(digit)

        return digit

    def between(self, minimum=1, maximum=1000):
        """Generate a random number between minimum and maximum.

        :param minimum: Minimum of range.
        :param maximum: Maximum of range
        :return: Number
        """
        num = self.random.randint(int(minimum), int(maximum))
        return num

    def rating(self, maximum=5.0):
        """Generate random rating for something.

        :param maximum: Minimum value (default is 5.0).
        :return: Rating.
        :rtype: float
        :Example:
            4.7
        """
        res = '{0:0.1f}'.format(self.random.uniform(0, float(maximum)))
        return float(res)



CS = 'cs'
DA = 'da'
DE = 'de'
DE_AT = 'de-at'
DE_CH = 'de_ch'
EN = 'en'
EN_AU = 'en-au'
EN_CA = 'en-ca'
EN_GB = 'en-gb'
ES = 'es'
ES_MX = 'es-mx'
FA = 'fa'
FI = 'fi'
FR = 'fr'
HU = 'hu'
IS = 'is'
IT = 'it'
JA = 'ja'
KO = 'ko'
NL = 'nl'
NL_BE = 'nl-be'
NO = 'no'
PL = 'pl'
PT = 'pt'
PT_BR = 'pt-br'
RU = 'ru'
SV = 'sv'
TR = 'tr'
UK = 'uk'
ZH = 'zh'



from mimesis.providers import BaseProvider
from mimesis.utils import pull


class Text(BaseProvider):
    """Class for generate text data, i.e text, lorem ipsum and another."""

    def __init__(self, *args, **kwargs):
        """
        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self.data = pull('text.json', self.locale)

    def alphabet(self, letter_case=None):
        """Get an alphabet for current locale.

        :param letter_case: Letter case.
        :return: Alphabet.
        :rtype: list
        """
        letter_case = 'uppercase' if \
            not letter_case else letter_case

        alpha = self.data['alphabet'][letter_case]
        return alpha

    def level(self):
        """Generate a random level of danger or something else.

        :return: Level.
        :Example:
            critical.
        """
        levels = self.data['level']
        return self.random.choice(levels)

    def text(self, quantity=5):
        """Generate the text.

        :param quantity: Quantity of sentences.
        :return: Text.
        :Example:
            Haskell is a standardized, general-purpose purely
            functional programming language, with non-strict semantics
            and strong static typing.
        """
        text = ''
        for _ in range(int(quantity)):
            text += ' ' + self.random.choice(self.data['text'])
        return text.strip()

    def sentence(self):
        """Get a random sentence from text.

        :return: Sentence.
        :Example:
            Any element of a tuple can be accessed in constant time.
        """
        return self.text(quantity=1)

    def title(self):
        """Get a random title.

        :return: The title.
        :Example:
            Erlang - is a general-purpose, concurrent,
            functional programming language.
        """
        return self.text(quantity=1)

    def words(self, quantity=5):
        """Get the random words.

        :param quantity: Quantity of words. Default is 5.
        :return: Word list.
        :Example:
            science, network, god, octopus, love.
        """
        words = self.data['words']['normal']
        words_list = [self.random.choice(words) for _ in range(int(quantity))]
        return words_list

    def word(self):
        """Get a random word.

        :return: Single word.
        :Example:
            Science.
        """
        return self.words(quantity=1)[0]

    def swear_word(self):
        """Get a random swear word.

        :return: Swear word.
        :Example:
            Damn.
        """
        bad_words = self.data['words']['bad']
        return self.random.choice(bad_words)

    def quote(self):
        """Get a random quote.

        :return: Quote from movie.
        :Example:
            "Bond... James Bond."
        """
        quotes = self.data['quotes']
        return self.random.choice(quotes)

    def color(self):
        """Get a random name of color.

        :return: Color name.
        :Example:
            Red.
        """
        colors = self.data['color']
        return self.random.choice(colors)

    def hex_color(self):
        """Generate a hex color.

        :return: Hex color code.
        :Example:
            #D8346B
        """
        letters = '0123456789ABCDEF'
        color_code = '#' + ''.join(self.random.sample(letters, 6))
        return color_code

    def answer(self):
        """Get a random answer in current language.

        :return: An answer.
        :rtype: str
        :Example:
            No
        """
        answers = self.data['answers']
        return self.random.choice(answers)



from mimesis.data import EXTENSIONS, MIME_TYPES
from mimesis.providers import BaseProvider


class File(BaseProvider):
    """Class for generate fake data for files."""

    def extension(self, file_type='text'):
        """Get a random file extension from list.

        :param file_type: File type (source, text, data, audio, video, image,
        executable, compressed).
        :return: Extension of a file.
        :Example:
            .py (file_type='source').
        """
        k = file_type.lower()
        return self.random.choice(EXTENSIONS[k])

    def mime_type(self, type_t='application'):
        """Get a random mime type from list.

        :return: Mime type.
        :param type_t: Type of media: (application,
        image, video, audio, text, message).
        :rtype: str
        """
        supported = ' '.join(MIME_TYPES.keys())

        if type_t not in list(MIME_TYPES.keys()):
            raise ValueError(
                'Unsupported mime type! Use: {}'.format(supported))

        mime_type = self.random.choice(MIME_TYPES[type_t])
        return mime_type



from mimesis.data import AIRPLANES, CARS, TRUCKS
from mimesis.providers import BaseProvider, Code


class Transport(BaseProvider):
    """Class that provides dummy data about transport."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self._model = Code().custom_code

    def truck(self, model_mask='#### @@'):
        """Generate a truck model.

        :param model_mask: Mask of truck model. Here '@' is a \
        placeholder of characters and '#' is a placeholder of digits.
        :return: Dummy truck model.
        :Example:
            Caledon-966O.
        """
        model = self._model(mask=model_mask)
        truck = self.random.choice(TRUCKS)
        return '%s-%s' % (truck, model)

    def car(self):
        """Get a random vehicle.

        :return: A vehicle.
        :Example:
            Tesla Model S.
        """
        return self.random.choice(CARS)

    def airplane(self, model_mask='###'):
        """Generate a dummy airplane model.

        :param model_mask: Mask of truck model. Here '@' is a \
        placeholder of characters and '#' is a placeholder of digits.
        :return:
        :Example:
            Boeing 727.
        """
        model = self._model(mask=model_mask)
        plane = self.random.choice(AIRPLANES)
        return '%s %s' % (plane, model)



from mimesis.data import SI_PREFIXES
from mimesis.exceptions import WrongArgument

from .base import BaseProvider


class UnitSystem(BaseProvider):
    """Class for generating name of unit.
    """

    @staticmethod
    def mass(symbol=False):
        """Get a mass unit name.

        :param symbol: Symbol of unit.
        :return: Mass unit name.
        :Example:
            gr
        """
        if not symbol:
            return 'gram'
        return 'gr'

    @staticmethod
    def information(symbol=False):
        if not symbol:
            return 'byte'
        return 'b'

    @staticmethod
    def thermodynamic_temperature(symbol=False):
        """Get the thermodynamic temperature unit name.

        :param symbol: Symbol of unit.
        :return: Thermodynamic temperature unit name
        :Example:
            K
        """
        if not symbol:
            return 'kelvin'
        return 'K'

    @staticmethod
    def amount_of_substance(symbol=False):
        """Get unit name of amount of substance.

        :param symbol: Symbol of unit.
        :return: Unit name of amount of substance.
        :Example:
            mol
        """
        if not symbol:
            return 'mole'
        return 'mol'

    @staticmethod
    def angle(symbol=False):
        """Get unit name of angle.

        :param symbol: Symbol of unit.
        :return: Unit name of angle.
        :Example:
            radian
        """
        if not symbol:
            return 'radian'
        return 'r'

    @staticmethod
    def solid_angle(symbol=False):
        """Get unit name if solid angle

        :param symbol: Symbol of unit.
        :return: Unit name of solid angle
        :Example:
            ㏛
        """
        if not symbol:
            return 'steradian'
        return '㏛'

    @staticmethod
    def frequency(symbol=False):
        """Get unit name of frequency.

        :param symbol: Symbol of unit.
        :return: Unit name if frequency.
        :Example:
            Hz
        """
        if not symbol:
            return 'hertz'
        return 'Hz'

    @staticmethod
    def force(symbol=False):
        """Get unit name of fore.

        :param symbol:  Symbol of unit.
        :return:  Unit name of force.
        :Example:
            N
        """
        if not symbol:
            return 'newton'
        return 'N'

    @staticmethod
    def pressure(symbol=False):
        """Get unit name of pressure.

        :param symbol: Symbol of unit.
        :return: Unit name of pressure.
        :Example:
            pascal
        """
        if not symbol:
            return 'pascal'
        return 'P'

    @staticmethod
    def energy(symbol=False):
        """Get unit name of energy.

        :param symbol: Symbol of unit.
        :return: Unit name of energy.
        :Example:
            J
        """
        if not symbol:
            return 'joule'
        return 'J'

    @staticmethod
    def power(symbol=False):
        """Get unit name of power.

        :param symbol: Symbol of unit.
        :return: Unit name of power.
        :Example:
            watt
        """
        if not symbol:
            return 'watt'
        return 'W'

    def flux(self, symbol=True):
        return self.power(symbol)

    @staticmethod
    def electric_charge(symbol=False):
        """Get unit name of electric charge.

        :param symbol: Symbol of unit.
        :return: Unit name of electric charge.
        :Example:
            coulomb
        """
        if not symbol:
            return 'coulomb'
        return 'C'

    @staticmethod
    def voltage(symbol=False):
        """Get unit name of voltage.

        :param symbol: Symbol of unit.
        :return: Unit name of voltage.
        :Example:
            volt
        """
        if not symbol:
            return 'volt'
        return 'V'

    @staticmethod
    def electric_capacitance(symbol=False):
        """Get unit name of electric capacitance.

        :param symbol: Symbol of unit.
        :return: Unit name of electric capacitance.
        :Example:
            F
        """
        if not symbol:
            return 'farad'
        return 'F'

    @staticmethod
    def electric_resistance(symbol=False):
        """Get name of electric resistance.

        :param symbol: Symbol of unit.
        :return: Name of electric resistance.
        :Example:
            Ω
        """
        if not symbol:
            return 'ohm'
        return 'Ω'

    def impedance(self, symbol=False):
        return self.electric_resistance(symbol)

    def reactance(self, symbol=False):
        return self.electric_resistance(symbol)

    @staticmethod
    def electrical_conductance(symbol=False):
        """Get unit name of electrical conductance.

        :param symbol: Symbol of unit.
        :return: Unit name of electrical conductance.
        :Example:
            siemens
        """
        if not symbol:
            return 'siemens'
        return 'S'

    @staticmethod
    def magnetic_flux(symbol=False):
        """Get unit name of magnetic flux.

        :param symbol: Symbol of unit.
        :return: Unit name of magnetic flux.
        :Example:
            Wb
        """
        if not symbol:
            return 'weber'
        return 'Wb'

    @staticmethod
    def magnetic_flux_density(symbol=False):
        """Get unit name of magnetic flux density.

        :param symbol: Symbol of unit.
        :return: Unit name of magnetic flux density.
        :Example:
            tesla
        """
        if not symbol:
            return 'tesla'
        return 'T'

    @staticmethod
    def inductance(symbol=False):
        """Get unit name of inductance.

        :param symbol: Symbol of unit.
        :return: Unit name of inductance.
        :Example:
            H
        """
        if not symbol:
            return 'henry'
        return 'H'

    @staticmethod
    def temperature(symbol=False):
        """Get unit name of temperature.

        :param symbol:
        :return:
        """
        if not symbol:
            return 'Celsius'
        return '°C'

    @staticmethod
    def radioactivity(symbol=False):
        """Get unit name of radioactivity.

        :param symbol: Symbol of unit.
        :return: Unit name of radioactivity.
        :Example:
            Bq
        """
        if not symbol:
            return 'becquerel'
        return 'Bq'

    def prefix(self, sign='positive', symbol=False):
        """Get a random prefix for the International System of Units (SI)

        :param sign: Sing of number (positive, negative)
        :param symbol: Return symbol of prefix.
        :return: Prefix for SI.
        :rtype: str
        :Example:
            mega
        """
        sign = sign.lower()

        if symbol:
            prefixes = SI_PREFIXES['_sym_']
        else:
            prefixes = SI_PREFIXES

        try:
            return self.random.choice(prefixes[sign])
        except KeyError:
            raise WrongArgument(
                "Unsupported sign. Use: 'positive' or 'negative'")



import inspect

from mimesis.providers import (Address, BaseProvider, Business, ClothingSizes,
                               Code, Datetime, Development, File, Food, Games,
                               Hardware, Internet, Numbers, Path, Personal,
                               Science, Text, Transport, UnitSystem,
                               Cryptographic)


class Generic(BaseProvider):
    """A lazy initialization of locale for all classes that have locales."""

    def __init__(self, locale):
        """
        :param locale: Current locale.
        """
        self.locale = locale
        self._personal = Personal
        self._address = Address
        self._datetime = Datetime
        self._business = Business
        self._text = Text
        self._food = Food
        self._science = Science
        self._code = Code
        self.unit_system = UnitSystem()
        self.file = File()
        self.numbers = Numbers()
        self.development = Development()
        self.hardware = Hardware()
        self.clothing_sizes = ClothingSizes()
        self.internet = Internet()
        self.transport = Transport()
        self.path = Path()
        self.games = Games()
        self.cryptographic = Cryptographic()

    def __getattr__(self, attrname):
        """Get _attribute without underscore

        :param attrname: Attribute name.
        :return: An attribute.
        """
        attribute = object.__getattribute__(self, '_' + attrname)
        if attribute and callable(attribute):
            return attribute(self.locale)

    def add_provider(self, cls):
        """Add a custom provider to Generic() object.

        :param cls: Custom provider.
        :return: None
        """
        if inspect.isclass(cls):
            name = ''
            if hasattr(cls, 'Meta'):
                if inspect.isclass(cls.Meta) and hasattr(cls.Meta, 'name'):
                    name = cls.Meta.name
            else:
                name = cls.__name__.lower()
            setattr(self, name, cls())
        else:
            raise TypeError('Provider must be a class')

    def add_providers(self, *providers):
        """Add a lot of custom providers to Generic() object.

        :param providers: Custom providers.
        :return: None
        """
        for provider in providers:
            self.add_provider(provider)



# Here you can see data that used in all locales. "int" is
# abbreviation of "International Data".

from .address import *
from .business import *
from .code import *
from .decorator import *
from .development import *
from .datetime import *
from .file import *
from .hardware import *
from .network import *
from .personal import *
from .scientific import *
from .transport import *
from .games import *



from .en import USASpecProvider
from .ja import JapanSpecProvider
from .ru import RussiaSpecProvider
from .pt_br import BrazilSpecProvider

__all__ = [
    'USASpecProvider',
    'JapanSpecProvider',
    'RussiaSpecProvider',
    'BrazilSpecProvider',
]



class UnsupportedLocale(KeyError):
    """An exception that raised when a locale doesn't supported."""


class JSONKeyError(KeyError):
    """An exception that raised when JSON key doesn't exist"""


class NotImplementedYet(NotImplementedError):
    """An exception that raised when NotImplementedError"""


class WrongArgument(KeyError):
    """An exception that raised when argument is wrong"""


class UnsupportedAlgorithm(AttributeError):
    """An exception that raised when user want to use unsupported algorithm"""


class UndefinedSchema(ValueError):
    """An exception that raised when schema if empty"""



BLOOD_GROUPS = (
    'O+',
    'A+',
    'B+',
    'AB+',
    'O−',
    'A−',
    'B−',
    'AB−',
)

FAVORITE_MUSIC_GENRE = (
    'Pop',
    'Rock',
    'Hard Rock',
    'Rhythm & Blues (R&B)',
    'Country',
    'Rock & Roll',
    'Soul',
    'Country Pop',
    'Pop Rock',
    'Heavy Metal',
    'Progressive Rock',
    'Alternative Rock',
    'Jazz',
    'New Wave',
    'Synthpop',
    'Hip Hop',
    'Folk',
    'New Age',
    'Blues Rock',
    'Ambient',
    'Ambient house',
    'Classic',
    'Neoclassic',
    'Drum and bass',
)

GENDER_SYMBOLS = (
    '♂',
    '♀',
    '⚲',
)

SEXUALITY_SYMBOLS = (
    '⚤',
    '⚢',
    '⚣',
    '⚥',
    '⚧',
    '⚪',
)

ENGLISH_LEVEL = [
    'Beginner',
    'Elementary',
    'Pre - Intermediate',
    'Intermediate',
    'Upper Intermediate',
    'Advanced',
    'Proficiency',
]



from mimesis.data import CONTINENT_CODES, COUNTRIES_ISO, SHORTENED_ADDRESS_FMT
from mimesis.providers import BaseProvider
from mimesis.utils import pull


class Address(BaseProvider):
    """Class for generate fake address data."""

    def __init__(self, *args, **kwargs):
        """
        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self.data = pull('address.json', self.locale)

    def street_number(self, maximum=1400):
        """Generate a random street number.

        :return: Street number.
        :Example:
            134.
        """
        number = self.random.randint(1, int(maximum))
        return '%s' % number

    def street_name(self):
        """Get a random street name.

        :return: Street name.
        :Example:
           Candlewood.
        """
        names = self.data['street']['name']
        return self.random.choice(names)

    def street_suffix(self):
        """Get a random street suffix.

        :return: Street suffix.
        :Example:
            Alley.
        """
        suffix = self.random.choice(self.data['street']['suffix'])
        return suffix

    def address(self):
        """Get a random full address (include Street number, suffix and name).

        :return: Full address.
        :Example:
            5 Central Sideline.
        """
        fmt = self.data['address_fmt']

        if self.locale in SHORTENED_ADDRESS_FMT:
            # Because fmt for ko is {st_name}{st_sfx} {st_num},
            # i.e not shortened address format
            if self.locale != 'ko':
                return fmt.format(
                    st_num=self.street_number(),
                    st_name=self.street_name(),
                )

        if self.locale == 'ja':
            cities = self.data['city']
            return fmt.format(
                city=self.random.choice(cities),
                n=self.random.randint(1, 100),
                nn=self.random.randint(1, 100),
                nnn=self.random.randint(1, 100),
            )

        return fmt.format(
            st_num=self.street_number(),
            st_name=self.street_name(),
            st_sfx=self.street_suffix(),

        )

    def state(self, abbr=False):
        """Get a random states or subject of country.

        :param abbr: If True then return ISO (ISO 3166-2)
        code of state/region/province/subject.
        :return: State of current country.
        :Example:
            Alabama (for locale `en`).
        """
        key = 'abbr' if abbr else 'name'
        states = self.data['state'][key]
        return self.random.choice(states)

    def postal_code(self):
        """Generate a postal code for current locale.

        :return: Postal code.
        :Example:
            389213
        """
        from mimesis.providers import Code

        mask = self.data['postal_code_fmt']
        return Code(self.locale).custom_code(mask)

    def country_iso(self, fmt='iso2'):
        """Get a random ISO code of country.

        :param fmt: Format of code (iso2, iso3, numeric).
        :return: ISO Code.
        :Example:
            DE
        """
        sup = ''.join(list(COUNTRIES_ISO.keys()))

        if fmt not in COUNTRIES_ISO:
            raise KeyError('Unsupported format. Use: {}'.format(sup))

        countries = COUNTRIES_ISO[fmt]
        return self.random.choice(countries)

    def country(self):
        """Get a random country.

        :return: The Country.
        :Example:
            Russia.
        """
        countries = self.data['country']['name']
        return self.random.choice(countries)

    def city(self):
        """Get a random city for current locale.

        :return: City name.
        :Example:
            Saint Petersburg.
        """
        cities = self.data['city']
        return self.random.choice(cities)

    def latitude(self):
        """Generate a random value of latitude (-90 to +90).

        :return: Value of longitude.
        :Example:
            -66.4214188124611
        """
        return self.random.uniform(-90, 90)

    def longitude(self):
        """
        Generate a random value of longitude (-180 to +180).

        :return: Value of longitude.
        :Example:
            112.18440260511943
        """
        return self.random.uniform(-180, 180)

    def coordinates(self):
        """Generate random geo coordinates.

        :return: Dict with coordinates.
        :rtype: dict
        :Example:
            {'latitude': 8.003968712834975, 'longitude': 36.02811153405548}
        """
        coord = {
            'longitude': self.longitude(),
            'latitude': self.latitude(),
        }
        return coord

    def continent(self, code=False):
        """Get a random continent name or continent
        code (code in international format)

        :return: Continent name.
        :Example:
            Africa (en)
        """
        if code:
            codes = CONTINENT_CODES
            return self.random.choice(codes)

        continent = self.random.choice(self.data['continent'])
        return continent



import datetime
from calendar import monthrange

from mimesis.data import ROMAN_NUMS
from mimesis.providers import BaseProvider
from mimesis.utils import pull


class Datetime(BaseProvider):
    """Class for generate the fake data that you can use for
    working with date and time."""

    def __init__(self, *args, **kwargs):
        """
        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self.data = pull('datetime.json', self.locale)

    def day_of_week(self, abbr=False):
        """Get a random day of week.

        :param abbr: Abbreviated name of the day.
        :return: Name of day of the week.
        :Example:
            Wednesday (Wed. when abbr=True).
        """
        key = 'abbr' if abbr else 'name'
        days = self.data['day'][key]
        return self.random.choice(days)

    def month(self, abbr=False):
        """Get a random month.

        :param abbr: if True then will be returned abbreviated month name.
        :return: Month name.
        :Example:
            January (Jan. when abbr=True).
        """
        key = 'abbr' if abbr else 'name'
        months = self.data['month'][key]
        return self.random.choice(months)

    def year(self, minimum=1990, maximum=2050):
        """Generate a random year.

        :param minimum: Minimum value.
        :param maximum: Maximum value
        :return: Year.
        :Example:
            2023.
        """
        return self.random.randint(int(minimum), int(maximum))

    def century(self):
        """Get a random value from list of centuries (roman format).

        :return: Century.
        :Example:
            XXI
        """
        return self.random.choice(ROMAN_NUMS)

    def periodicity(self):
        """Get a random periodicity string.

        :return: Periodicity.
        :Example:
            Never.
        """
        periodicity = self.data['periodicity']
        return self.random.choice(periodicity)

    def date(self, start=2000, end=2035, fmt=None):
        """Generate a string representing of random date formatted for
        the locale or as specified.

        :param start: Minimum value of year.
        :param end: Maximum value of year.
        :param fmt: Format string for date.
        :return: Formatted date.
        :Example:
            08/16/88 (en)
        """
        if not fmt:
            fmt = self.data['formats']['date']

        year = self.random.randint(start, end)
        month = self.random.randint(1, 12)
        d = datetime.date(
            year, month, self.random.randint(1, monthrange(year, month)[1]))
        return d.strftime(fmt)

    def time(self, fmt=None):
        """Generate a random time formatted for the locale or as specified.

        :return: Time.
        :Example:
            21:30:00 (en)
        """
        if not fmt:
            fmt = self.data['formats']['time']

        t = datetime.time(
            self.random.randint(0, 23),
            self.random.randint(0, 59),
            self.random.randint(0, 59),
            self.random.randint(0, 999999),
        )
        return t.strftime(fmt)

    def day_of_month(self):
        """Generate a random day of month, from 1 to 31.

        :return: Random value from 1 to 31.
        :Example:
            23
        """
        return self.random.randint(1, 31)



from .base import BaseProvider
from .address import Address
from .business import Business
from .clothing import ClothingSizes
from .code import Code
from .date import Datetime
from .development import Development
from .file import File
from .food import Food
from .hardware import Hardware
from .internet import Internet
from .numbers import Numbers
from .path import Path
from .personal import Personal
from .science import Science
from .structured import Structured
from .text import Text
from .transport import Transport
from .units import UnitSystem
from .games import Games
from .cryptographic import Cryptographic
from .generic import Generic

__all__ = [
    # The main class:
    'BaseProvider',

    # And data providers:
    'Address',
    'Business',
    'ClothingSizes',
    'Code',
    'Datetime',
    'Development',
    'File',
    'Food',
    'Games',
    'Hardware',
    'Internet',
    'Numbers',
    'Path',
    'Personal',
    'Science',
    'Structured',
    'Text',
    'Transport',
    'UnitSystem',
    'Cryptographic',

    # Has all:
    'Generic',
]



HTTP_METHODS = [
    'GET',
    'POST',
    'PUT',
    'DELETE',
    'OPTIONS',
]

HTTP_STATUS_CODES = [
    '100 Continue',
    '101 Switching Protocols',
    '200 OK',
    '201 Created',
    '202 Accepted',
    '203 Non Authoritative Information',
    '204 No Content',
    '205 Reset Content',
    '206 Partial Content',
    '207 Multi Status',
    '300 Multiple Choices',
    '301 Moved Permanently',
    '302 Found',
    '303 See Other',
    '304 Not Modified',
    '305 Use Proxy',
    '306 Reserved',
    '307 Temporary Redirect',
    '400 Bad Request',
    '401 Unauthorized',
    '402 Payment Required',
    '403 Forbidden',
    '404 Not Found',
    '405 Method Not Allowed',
    '406 Not Acceptable',
    '407 Proxy Authentication Required',
    '408 Request Timeout',
    '409 Conflict',
    '410 Gone',
    '411 Length Required',
    '412 Precondition Failed',
    '413 Request Entity Too Large',
    '414 Request Uri Too Long',
    '415 Unsupported Media Type',
    '416 Requested Range Not Satisfiable',
    '417 Expectation Failed',
    '422 Unprocessable Entity',
    '423 Locked',
    '424 Failed Dependency',
    '428 Precondition Required',
    '429 Too Many Requests',
    '431 Request Header Fields Too Large',
    '451 Unavailable For Legal Reasons',
    '500 Internal Server Error',
    '501 Not Implemented',
    '502 Bad Gateway',
    '503 Service Unavailable',
    '504 Gateway Timeout',
    '505 Version Not Supported',
    '507 Insufficient Storage',
    '511 Network Authentication Required',
]

SUBREDDITS = [
    '/r/AskReddit',
    '/r/IAmA',
    '/r/bestof',
    '/r/fatpeoplestories',
    '/r/pettyrevenge',
    '/r/TalesFromRetail',
    '/r/DoesAnybodyElse',
    '/r/CrazyIdeas',
    '/r/WTF',
    '/r/aww',
    '/r/cringepics',
    '/r/cringe',
    '/r/JusticePorn',
    '/r/MorbidReality',
    '/r/rage',
    '/r/mildlyinfuriating',
    '/r/creepy',
    '/r/creepyPMs',
    '/r/nosleep',
    '/r/nostalgia',
    '/r/gaming',
    '/r/leagueoflegends',
    '/r/pokemon',
    '/r/Minecraft',
    '/r/starcraft',
    '/r/Games',
    '/r/DotA2',
    '/r/skyrim',
    '/r/tf2',
    '/r/magicTCG',
    '/r/wow',
    '/r/KerbalSpaceProgram',
    '/r/mindcrack',
    '/r/Fallout',
    '/r/roosterteeth',
    '/r/Planetside',
    '/r/gamegrumps',
    '/r/battlefield3',
    '/r/zelda',
    '/r/darksouls',
    '/r/masseffect',
    '/r/arresteddevelopment',
    '/r/gameofthrones',
    '/r/doctorwho',
    '/r/mylittlepony',
    '/r/community',
    '/r/breakingbad',
    '/r/adventuretime',
    '/r/startrek',
    '/r/TheSimpsons',
    '/r/futurama',
    '/r/HIMYM',
    '/r/DunderMifflin',
    '/r/thewalkingdead',
    '/r/Music',
    '/r/movies',
    '/r/harrypotter',
    '/r/StarWars',
    '/r/DaftPunk',
    '/r/hiphopheads',
    '/r/anime',
    '/r/comicbooks',
    '/r/geek',
    '/r/batman',
    '/r/TheLastAirbender',
    '/r/Naruto',
    '/r/FanTheories',
    '/r/funny',
    '/r/AdviceAnimals',
    '/r/fffffffuuuuuuuuuuuu',
    '/r/4chan',
    '/r/ImGoingToHellForThis',
    '/r/firstworldanarchists',
    '/r/circlejerk',
    '/r/MURICA',
    '/r/facepalm',
    '/r/Jokes',
    '/r/wheredidthesodago',
    '/r/polandball',
    '/r/TrollXChromosomes',
    '/r/comics',
    '/r/nottheonion',
    '/r/britishproblems',
    '/r/TumblrInAction',
    '/r/onetruegod',
    '/r/pics',
    '/r/videos',
    '/r/gifs',
    '/r/reactiongifs',
    '/r/mildlyinteresting',
    '/r/woahdude',
    '/r/FiftyFifty',
    '/r/FoodPorn',
    '/r/HistoryPorn',
    '/r/wallpapers',
    '/r/youtubehaiku',
    '/r/Unexpected',
    '/r/photoshopbattles',
    '/r/AnimalsBeingJerks',
    '/r/cosplay',
    '/r/EarthPorn',
    '/r/QuotesPorn',
    '/r/awwnime',
    '/r/AbandonedPorn',
    '/r/carporn',
    '/r/PerfectTiming',
    '/r/OldSchoolCool',
    '/r/RoomPorn',
    '/r/Pareidolia',
    '/r/MapPorn',
    '/r/tumblr',
    '/r/techsupportgore',
    '/r/PrettyGirls',
    '/r/itookapicture',
    '/r/todayilearned',
    '/r/science',
    '/r/askscience',
    '/r/space',
    '/r/AskHistorians',
    '/r/YouShouldKnow',
    '/r/explainlikeimfive',
    '/r/trees',
    '/r/MakeupAddiction',
    '/r/cats',
    '/r/LifeProTips',
    '/r/RedditLaqueristas',
    '/r/Random_Acts_Of_Amazon',
    '/r/food',
    '/r/guns',
    '/r/tattoos',
    '/r/corgi',
    '/r/teenagers',
    '/r/GetMotivated',
    '/r/motorcycles',
    '/r/sex',
    '/r/progresspics',
    '/r/DIY',
    '/r/bicycling',
    '/r/Fitness',
    '/r/lifehacks',
    '/r/longboarding',
    '/r/Frugal',
    '/r/drunk',
    '/r/Art',
    '/r/loseit',
]

SUBREDDITS_NSFW = [
    '/r/nsfw',
    '/r/OnOff',
    '/r/nsfwhardcore',
    '/r/Hotchickswithtattoos',
    '/r/randomsexiness',
    '/r/HappyEmbarrassedGirls',
    '/r/suicidegirls',
    '/r/nsfw2',
    '/r/nsfwcosplay',
    '/r/nsfwoutfits',
    '/r/unashamed',
    '/r/Camwhores',
    '/r/TipOfMyPenis',
    '/r/bonermaterial',
    '/r/Shemales',
    '/r/volleyballgirls',
    '/r/passionx',
    '/r/voluptuous',
    '/r/porn',
    '/r/fitgirls',
    '/r/SheLikesItRough',
    '/r/sexyfrex',
    '/r/trashyboners',
    '/r/GirlswithNeonHair',
    '/r/nsfw411',
    '/r/iWantToFuckHer',
    '/r/freeuse',
    '/r/exxxtras',
    '/r/distension',
    '/r/twingirls',
    '/r/ChangingRooms',
    '/r/realgirls',
    '/r/amateur',
    '/r/homemadexxx',
    '/r/AmateurArchives',
    '/r/dirtypenpals',
    '/r/wifesharing',
    '/r/FestivalSluts',
    '/r/hotwife',
    '/r/NSFW_Snapchat',
    '/r/CollegeAmateurs',
    '/r/amateurcumsluts',
    '/r/GoneWild',
    '/r/gonewildcurvy',
    '/r/AsiansGoneWild',
    '/r/GWCouples',
    '/r/GoneWildplus',
    '/r/PetiteGoneWild',
    '/r/gonewildstories',
    '/r/GoneWildTube',
    '/r/treesgonewild',
    '/r/gonewildaudio',
    '/r/workgonewild',
    '/r/GWNerdy',
    '/r/BigBoobsGW',
    '/r/gonemild',
    '/r/altgonewild',
    '/r/gwcumsluts',
    '/r/gonewildcolor',
    '/r/gifsgonewild',
    '/r/analgw',
    '/r/rule34',
    '/r/hentai',
    '/r/rule34_comics',
    '/r/AsianHotties',
    '/r/AsianHotties',
    '/r/AsiansGoneWild',
    '/r/realasians',
    '/r/juicyasians',
    '/r/IndianBabes',
    '/r/NSFW_Japan',
    '/r/ass',
    '/r/girlsinyogapants',
    '/r/asstastic',
    '/r/facedownassup',
    '/r/anal',
    '/r/assinthong',
    '/r/analgw',
    '/r/BDSM',
    '/r/Bondage',
    '/r/blowjobs',
    '/r/lipsthatgrip',
    '/r/boobies',
    '/r/BustyPetite',
    '/r/tinytits',
    '/r/TittyDrop',
    '/r/burstingout',
    '/r/hugeboobs',
    '/r/stacked',
    '/r/boltedontits',
    '/r/BigBoobsGW',
    '/r/boobbounce',
    '/r/boobs',
    '/r/downblouse',
    '/r/celebnsfw',
    '/r/WatchItForThePlot',
    '/r/girlsinyogapants',
    '/r/girlswithglasses',
    '/r/lingerie',
    '/r/stockings',
    '/r/candidfashionpolice',
    '/r/WtSSTaDaMiT',
    '/r/tightdresses',
    '/r/upskirt',
    '/r/cumsluts',
    '/r/GirlsFinishingTheJob',
    '/r/cumfetish',
    '/r/creampies',
    '/r/amateurcumsluts',
    '/r/gonewildcurvy',
    '/r/curvy',
    '/r/gonewildplus',
    '/r/thick',
    '/r/juicyasians',
    '/r/NSFW_GIF',
    '/r/nsfw_gifs',
    '/r/porn_gifs',
    '/r/porninfifteenseconds',
    '/r/CuteModeSlutMode',
    '/r/ginger',
    '/r/redheads',
    '/r/60fpsporn',
    '/r/highresNSFW',
    '/r/NSFW_HTML5',
    '/r/datgap',
    '/r/girlsinyogapants',
    '/r/stockings',
    '/r/lesbians',
    '/r/StraightGirlsPlaying',
    '/r/ladybonersgw',
    '/r/milf',
    '/r/BustyPetite',
    '/r/dirtysmall',
    '/r/petitegonewild',
    '/r/pussy',
    '/r/rearpussy',
    '/r/innie',
    '/r/legalteens',
    '/r/collegesluts',
    '/r/pornvids',
    '/r/nsfw_videos',
    '/r/palegirls',
    '/r/pawg',
    '/r/holdthemoan',
    '/r/O_faces',
    '/r/grool',
    '/r/jilling',
    '/r/gettingherselfoff',
    '/r/quiver',
]

DOMAINS = [
    '.ac',
    '.ad',
    '.biz',
    '.by',
    '.com',
    '.edu',
    '.gov',
    '.in',
    '.info',
    '.int',
    '.io',
    '.is',
    '.mil',
    '.name',
    '.net',
    '.org',
    '.place',
    '.pw',
    '.ru',
    '.ua',
    '.uk',
    '.us',
    '.uk',
]

EMAIL_DOMAINS = (
    '@gmail.com',
    '@yandex.com',
    '@yahoo.com',
    '@live.com',
    '@outlook.com',
)
EMOJI = (
    ':bowtie:',
    ':smile:',
    ':laughing:',
    ':blush:',
    ':smiley:',
    ':relaxed:',
    ':smirk:',
    ':heart_eyes:',
    ':kissing_heart:',
    ':kissing_closed_eyes:',
    ':flushed:',
    ':relieved:',
    ':satisfied:',
    ':grin:',
    ':wink:',
    ':stuck_out_tongue_winking_eye:',
    ':stuck_out_tongue_closed_eyes:',
    ':grinning:',
    ':kissing:',
    ':kissing_smiling_eyes:',
    ':stuck_out_tongue:',
    ':sleeping:',
    ':worried:',
    ':frowning:',
    ':anguished:',
    ':open_mouth:',
    ':grimacing:',
    ':confused:',
    ':hushed:',
    ':expressionless:',
    ':unamused:',
    ':sweat_smile:',
    ':sweat:',
    ':disappointed_relieved:',
    ':weary:',
    ':pensive:',
    ':disappointed:',
    ':confounded:',
    ':fearful:',
    ':cold_sweat:',
    ':persevere:',
    ':cry:',
    ':sob:',
    ':joy:',
    ':astonished:',
    ':scream:',
    ':neckbeard:',
    ':tired_face:',
    ':angry:',
    ':rage:',
    ':triumph:',
    ':sleepy:',
    ':yum:',
    ':mask:',
    ':sunglasses:',
    ':dizzy_face:',
    ':imp:',
    ':smiling_imp:',
    ':neutral_face:',
    ':no_mouth:',
    ':innocent:',
    ':alien:',
    ':yellow_heart:',
    ':blue_heart:',
    ':purple_heart:',
    ':heart:',
    ':green_heart:',
    ':broken_heart:',
    ':heartbeat:',
    ':heartpulse:',
    ':two_hearts:',
    ':revolving_hearts:',
    ':cupid:',
    ':sparkling_heart:',
    ':sparkles:',
    ':star:',
    ':star2:',
    ':dizzy:',
    ':boom:',
    ':collision:',
    ':anger:',
    ':exclamation:',
    ':question:',
    ':grey_exclamation:',
    ':grey_question:',
    ':zzz:',
    ':dash:',
    ':sweat_drops:',
    ':notes:',
    ':musical_note:',
    ':fire:',
    ':hankey:',
    ':poop:',
    ':shit:',
    ':+1:',
    ':thumbsup:',
    ':-1:',
    ':thumbsdown:',
    ':ok_hand:',
    ':punch:',
    ':facepunch:',
    ':fist:',
    ':v:',
    ':wave:',
    ':hand:',
    ':raised_hand:',
    ':open_hands:',
    ':point_up:',
    ':point_down:',
    ':point_left:',
    ':point_right:',
    ':raised_hands:',
    ':pray:',
    ':point_up_2:',
    ':clap:',
    ':muscle:',
    ':metal:',
    ':fu:',
    ':runner:',
    ':running:',
    ':couple:',
    ':family:',
    ':two_men_holding_hands:',
    ':two_women_holding_hands:',
    ':dancer:',
    ':dancers:',
    ':ok_woman:',
    ':no_good:',
    ':information_desk_person:',
    ':raising_hand:',
    ':bride_with_veil:',
    ':person_with_pouting_face:',
    ':person_frowning:',
    ':bow:',
    ':couplekiss:',
    ':couple_with_heart:',
    ':massage:',
    ':haircut:',
    ':nail_care:',
    ':boy:',
    ':girl:',
    ':woman:',
    ':man:',
    ':baby:',
    ':older_woman:',
    ':older_man:',
    ':person_with_blond_hair:',
    ':man_with_gua_pi_mao:',
    ':man_with_turban:',
    ':construction_worker:',
    ':cop:',
    ':angel:',
    ':princess:',
    ':smiley_cat:',
    ':smile_cat:',
    ':heart_eyes_cat:',
    ':kissing_cat:',
    ':smirk_cat:',
    ':scream_cat:',
    ':crying_cat_face:',
    ':joy_cat:',
    ':pouting_cat:',
    ':japanese_ogre:',
    ':japanese_goblin:',
    ':see_no_evil:',
    ':hear_no_evil:',
    ':speak_no_evil:',
    ':guardsman:',
    ':skull:',
    ':feet:',
    ':lips:',
    ':kiss:',
    ':droplet:',
    ':ear:',
    ':eyes:',
    ':nose:',
    ':tongue:',
    ':love_letter:',
    ':bust_in_silhouette:',
    ':busts_in_silhouette:',
    ':speech_balloon:',
    ':thought_balloon:',
    ':feelsgood:',
    ':finnadie:',
    ':goberserk:',
    ':godmode:',
    ':hurtrealbad:',
    ':rage1:',
    ':rage2:',
    ':rage3:',
    ':rage4:',
    ':suspect:',
    ':trollface:',
    ':sunny:',
    ':umbrella:',
    ':cloud:',
    ':snowflake:',
    ':snowman:',
    ':zap:',
    ':cyclone:',
    ':foggy:',
    ':ocean:',
    ':cat:',
    ':dog:',
    ':mouse:',
    ':hamster:',
    ':rabbit:',
    ':wolf:',
    ':frog:',
    ':tiger:',
    ':koala:',
    ':bear:',
    ':pig:',
    ':pig_nose:',
    ':cow:',
    ':boar:',
    ':monkey_face:',
    ':monkey:',
    ':horse:',
    ':racehorse:',
    ':camel:',
    ':sheep:',
    ':elephant:',
    ':panda_face:',
    ':snake:',
    ':bird:',
    ':baby_chick:',
    ':hatched_chick:',
    ':hatching_chick:',
    ':chicken:',
    ':penguin:',
    ':turtle:',
    ':bug:',
    ':honeybee:',
    ':ant:',
    ':beetle:',
    ':snail:',
    ':octopus:',
    ':tropical_fish:',
    ':fish:',
    ':whale:',
    ':whale2:',
    ':dolphin:',
    ':cow2:',
    ':ram:',
    ':rat:',
    ':water_buffalo:',
    ':tiger2:',
    ':rabbit2:',
    ':dragon:',
    ':goat:',
    ':rooster:',
    ':dog2:',
    ':pig2:',
    ':mouse2:',
    ':ox:',
    ':dragon_face:',
    ':blowfish:',
    ':crocodile:',
    ':dromedary_camel:',
    ':leopard:',
    ':cat2:',
    ':poodle:',
    ':paw_prints:',
    ':bouquet:',
    ':cherry_blossom:',
    ':tulip:',
    ':four_leaf_clover:',
    ':rose:',
    ':sunflower:',
    ':hibiscus:',
    ':maple_leaf:',
    ':leaves:',
    ':fallen_leaf:',
    ':herb:',
    ':mushroom:',
    ':cactus:',
    ':palm_tree:',
    ':evergreen_tree:',
    ':deciduous_tree:',
    ':chestnut:',
    ':seedling:',
    ':blossom:',
    ':ear_of_rice:',
    ':shell:',
    ':globe_with_meridians:',
    ':sun_with_face:',
    ':full_moon_with_face:',
    ':new_moon_with_face:',
    ':new_moon:',
    ':waxing_crescent_moon:',
    ':first_quarter_moon:',
    ':waxing_gibbous_moon:',
    ':full_moon:',
    ':waning_gibbous_moon:',
    ':last_quarter_moon:',
    ':waning_crescent_moon:',
    ':last_quarter_moon_with_face:',
    ':first_quarter_moon_with_face:',
    ':crescent_moon:',
    ':earth_africa:',
    ':earth_americas:',
    ':earth_asia:',
    ':volcano:',
    ':milky_way:',
    ':partly_sunny:',
    ':octocat:',
    ':squirrel:',
    ':bamboo:',
    ':gift_heart:',
    ':dolls:',
    ':school_satchel:',
    ':mortar_board:',
    ':flags:',
    ':fireworks:',
    ':sparkler:',
    ':wind_chime:',
    ':rice_scene:',
    ':jack_o_lantern:',
    ':ghost:',
    ':santa:',
    ':christmas_tree:',
    ':gift:',
    ':bell:',
    ':no_bell:',
    ':tanabata_tree:',
    ':tada:',
    ':confetti_ball:',
    ':balloon:',
    ':crystal_ball:',
    ':cd:',
    ':dvd:',
    ':floppy_disk:',
    ':camera:',
    ':video_camera:',
    ':movie_camera:',
    ':computer:',
    ':tv:',
    ':iphone:',
    ':phone:',
    ':telephone:',
    ':telephone_receiver:',
    ':pager:',
    ':fax:',
    ':minidisc:',
    ':vhs:',
    ':sound:',
    ':speaker:',
    ':mute:',
    ':loudspeaker:',
    ':mega:',
    ':hourglass:',
    ':hourglass_flowing_sand:',
    ':alarm_clock:',
    ':watch:',
    ':radio:',
    ':satellite:',
    ':loop:',
    ':mag:',
    ':mag_right:',
    ':unlock:',
    ':lock:',
    ':lock_with_ink_pen:',
    ':closed_lock_with_key:',
    ':key:',
    ':bulb:',
    ':flashlight:',
    ':high_brightness:',
    ':low_brightness:',
    ':electric_plug:',
    ':battery:',
    ':calling:',
    ':email:',
    ':mailbox:',
    ':postbox:',
    ':bath:',
    ':bathtub:',
    ':shower:',
    ':toilet:',
    ':wrench:',
    ':nut_and_bolt:',
    ':hammer:',
    ':seat:',
    ':moneybag:',
    ':yen:',
    ':dollar:',
    ':pound:',
    ':euro:',
    ':credit_card:',
    ':money_with_wings:',
    ':e-mail:',
    ':inbox_tray:',
    ':outbox_tray:',
    ':envelope:',
    ':incoming_envelope:',
    ':postal_horn:',
    ':mailbox_closed:',
    ':mailbox_with_mail:',
    ':mailbox_with_no_mail:',
    ':package:',
    ':door:',
    ':smoking:',
    ':bomb:',
    ':gun:',
    ':hocho:',
    ':pill:',
    ':syringe:',
    ':page_facing_up:',
    ':page_with_curl:',
    ':bookmark_tabs:',
    ':bar_chart:',
    ':chart_with_upwards_trend:',
    ':chart_with_downwards_trend:',
    ':scroll:',
    ':clipboard:',
    ':calendar:',
    ':date:',
    ':card_index:',
    ':file_folder:',
    ':open_file_folder:',
    ':scissors:',
    ':pushpin:',
    ':paperclip:',
    ':black_nib:',
    ':pencil2:',
    ':straight_ruler:',
    ':triangular_ruler:',
    ':closed_book:',
    ':green_book:',
    ':blue_book:',
    ':orange_book:',
    ':notebook:',
    ':notebook_with_decorative_cover:',
    ':ledger:',
    ':books:',
    ':bookmark:',
    ':name_badge:',
    ':microscope:',
    ':telescope:',
    ':newspaper:',
    ':football:',
    ':basketball:',
    ':soccer:',
    ':baseball:',
    ':tennis:',
    ':8ball:',
    ':rugby_football:',
    ':bowling:',
    ':golf:',
    ':mountain_bicyclist:',
    ':bicyclist:',
    ':horse_racing:',
    ':snowboarder:',
    ':swimmer:',
    ':surfer:',
    ':ski:',
    ':spades:',
    ':hearts:',
    ':clubs:',
    ':diamonds:',
    ':gem:',
    ':ring:',
    ':trophy:',
    ':musical_score:',
    ':musical_keyboard:',
    ':violin:',
    ':space_invader:',
    ':video_game:',
    ':black_joker:',
    ':flower_playing_cards:',
    ':game_die:',
    ':dart:',
    ':mahjong:',
    ':clapper:',
    ':memo:',
    ':pencil:',
    ':book:',
    ':art:',
    ':microphone:',
    ':headphones:',
    ':trumpet:',
    ':saxophone:',
    ':guitar:',
    ':shoe:',
    ':sandal:',
    ':high_heel:',
    ':lipstick:',
    ':boot:',
    ':shirt:',
    ':tshirt:',
    ':necktie:',
    ':womans_clothes:',
    ':dress:',
    ':running_shirt_with_sash:',
    ':jeans:',
    ':kimono:',
    ':bikini:',
    ':ribbon:',
    ':tophat:',
    ':crown:',
    ':womans_hat:',
    ':mans_shoe:',
    ':closed_umbrella:',
    ':briefcase:',
    ':handbag:',
    ':pouch:',
    ':purse:',
    ':eyeglasses:',
    ':fishing_pole_and_fish:',
    ':coffee:',
    ':tea:',
    ':sake:',
    ':baby_bottle:',
    ':beer:',
    ':beers:',
    ':cocktail:',
    ':tropical_drink:',
    ':wine_glass:',
    ':fork_and_knife:',
    ':pizza:',
    ':hamburger:',
    ':fries:',
    ':poultry_leg:',
    ':meat_on_bone:',
    ':spaghetti:',
    ':curry:',
    ':fried_shrimp:',
    ':bento:',
    ':sushi:',
    ':fish_cake:',
    ':rice_ball:',
    ':rice_cracker:',
    ':rice:',
    ':ramen:',
    ':stew:',
    ':oden:',
    ':dango:',
    ':egg:',
    ':bread:',
    ':doughnut:',
    ':custard:',
    ':icecream:',
    ':ice_cream:',
    ':shaved_ice:',
    ':birthday:',
    ':cake:',
    ':cookie:',
    ':chocolate_bar:',
    ':candy:',
    ':lollipop:',
    ':honey_pot:',
    ':apple:',
    ':green_apple:',
    ':tangerine:',
    ':lemon:',
    ':cherries:',
    ':grapes:',
    ':watermelon:',
    ':strawberry:',
    ':peach:',
    ':melon:',
    ':banana:',
    ':pear:',
    ':pineapple:',
    ':sweet_potato:',
    ':eggplant:',
    ':tomato:',
    ':corn:',
    ':house:',
    ':house_with_garden:',
    ':school:',
    ':office:',
    ':post_office:',
    ':hospital:',
    ':bank:',
    ':convenience_store:',
    ':love_hotel:',
    ':hotel:',
    ':wedding:',
    ':mimesis:',
    ':department_store:',
    ':european_post_office:',
    ':city_sunrise:',
    ':city_sunset:',
    ':japanese_castle:',
    ':european_castle:',
    ':tent:',
    ':factory:',
    ':tokyo_tower:',
    ':japan:',
    ':mount_fuji:',
    ':sunrise_over_mountains:',
    ':sunrise:',
    ':stars:',
    ':statue_of_liberty:',
    ':bridge_at_night:',
    ':carousel_horse:',
    ':rainbow:',
    ':ferris_wheel:',
    ':fountain:',
    ':roller_coaster:',
    ':ship:',
    ':speedboat:',
    ':boat:',
    ':sailboat:',
    ':rowboat:',
    ':anchor:',
    ':rocket:',
    ':airplane:',
    ':helicopter:',
    ':steam_locomotive:',
    ':tram:',
    ':mountain_railway:',
    ':bike:',
    ':aerial_tramway:',
    ':suspension_railway:',
    ':mountain_cableway:',
    ':tractor:',
    ':blue_car:',
    ':oncoming_automobile:',
    ':car:',
    ':red_car:',
    ':taxi:',
    ':oncoming_taxi:',
    ':articulated_lorry:',
    ':bus:',
    ':oncoming_bus:',
    ':rotating_light:',
    ':police_car:',
    ':oncoming_police_car:',
    ':fire_engine:',
    ':ambulance:',
    ':minibus:',
    ':truck:',
    ':train:',
    ':station:',
    ':train2:',
    ':bullettrain_front:',
    ':bullettrain_side:',
    ':light_rail:',
    ':monorail:',
    ':railway_car:',
    ':trolleybus:',
    ':ticket:',
    ':fuelpump:',
    ':vertical_traffic_light:',
    ':traffic_light:',
    ':warning:',
    ':construction:',
    ':beginner:',
    ':atm:',
    ':slot_machine:',
    ':busstop:',
    ':barber:',
    ':hotsprings:',
    ':checkered_flag:',
    ':crossed_flags:',
    ':izakaya_lantern:',
    ':moyai:',
    ':circus_tent:',
    ':performing_arts:',
    ':round_pushpin:',
    ':triangular_flag_on_post:',
    ':jp:',
    ':kr:',
    ':cn:',
    ':us:',
    ':fr:',
    ':es:',
    ':it:',
    ':ru:',
    ':gb:',
    ':uk:',
    ':de:',
    ':one:',
    ':two:',
    ':three:',
    ':four:',
    ':five:',
    ':six:',
    ':seven:',
    ':eight:',
    ':nine:',
    ':keycap_ten:',
    ':1234:',
    ':zero:',
    ':hash:',
    ':symbols:',
    ':arrow_backward:',
    ':arrow_down:',
    ':arrow_forward:',
    ':arrow_left:',
    ':capital_abcd:',
    ':abcd:',
    ':abc:',
    ':arrow_lower_left:',
    ':arrow_lower_right:',
    ':arrow_right:',
    ':arrow_up:',
    ':arrow_upper_left:',
    ':arrow_upper_right:',
    ':arrow_double_down:',
    ':arrow_double_up:',
    ':arrow_down_small:',
    ':arrow_heading_down:',
    ':arrow_heading_up:',
    ':leftwards_arrow_with_hook:',
    ':arrow_right_hook:',
    ':left_right_arrow:',
    ':arrow_up_down:',
    ':arrow_up_small:',
    ':arrows_clockwise:',
    ':arrows_counterclockwise:',
    ':rewind:',
    ':fast_forward:',
    ':information_source:',
    ':ok:',
    ':twisted_rightwards_arrows:',
    ':repeat:',
    ':repeat_one:',
    ':new:',
    ':top:',
    ':up:',
    ':cool:',
    ':free:',
    ':ng:',
    ':cinema:',
    ':koko:',
    ':signal_strength:',
    ':u5272:',
    ':u5408:',
    ':u55b6:',
    ':u6307:',
    ':u6708:',
    ':u6709:',
    ':u6e80:',
    ':u7121:',
    ':u7533:',
    ':u7a7a:',
    ':u7981:',
    ':sa:',
    ':restroom:',
    ':mens:',
    ':womens:',
    ':baby_symbol:',
    ':no_smoking:',
    ':parking:',
    ':wheelchair:',
    ':metro:',
    ':baggage_claim:',
    ':accept:',
    ':wc:',
    ':potable_water:',
    ':put_litter_in_its_place:',
    ':secret:',
    ':congratulations:',
    ':m:',
    ':passport_control:',
    ':left_luggage:',
    ':customs:',
    ':ideograph_advantage:',
    ':cl:',
    ':sos:',
    ':id:',
    ':no_entry_sign:',
    ':underage:',
    ':no_mobile_phones:',
    ':do_not_litter:',
    ':non-potable_water:',
    ':no_bicycles:',
    ':no_pedestrians:',
    ':children_crossing:',
    ':no_entry:',
    ':eight_spoked_asterisk:',
    ':sparkle:',
    ':eight_pointed_black_star:',
    ':heart_decoration:',
    ':vs:',
    ':vibration_mode:',
    ':mobile_phone_off:',
    ':chart:',
    ':currency_exchange:',
    ':aries:',
    ':taurus:',
    ':gemini:',
    ':cancer:',
    ':leo:',
    ':virgo:',
    ':libra:',
    ':scorpius:',
    ':sagittarius:',
    ':capricorn:',
    ':aquarius:',
    ':pisces:',
    ':ophiuchus:',
    ':six_pointed_star:',
    ':negative_squared_cross_mark:',
    ':a:',
    ':b:',
    ':ab:',
    ':o2:',
    ':diamond_shape_with_a_dot_inside:',
    ':recycle:',
    ':end:',
    ':back:',
    ':on:',
    ':soon:',
    ':clock1:',
    ':clock130:',
    ':clock10:',
    ':clock1030:',
    ':clock11:',
    ':clock1130:',
    ':clock12:',
    ':clock1230:',
    ':clock2:',
    ':clock230:',
    ':clock3:',
    ':clock330:',
    ':clock4:',
    ':clock430:',
    ':clock5:',
    ':clock530:',
    ':clock6:',
    ':clock630:',
    ':clock7:',
    ':clock730:',
    ':clock8:',
    ':clock830:',
    ':clock9:',
    ':clock930:',
    ':heavy_dollar_sign:',
    ':copyright:',
    ':registered:',
    ':tm:',
    ':x:',
    ':heavy_exclamation_mark:',
    ':bangbang:',
    ':interrobang:',
    ':o:',
    ':heavy_multiplication_x:',
    ':heavy_plus_sign:',
    ':heavy_minus_sign:',
    ':heavy_division_sign:',
    ':white_flower:',
    ':100:',
    ':heavy_check_mark:',
    ':ballot_box_with_check:',
    ':radio_button:',
    ':link:',
    ':curly_loop:',
    ':wavy_dash:',
    ':part_alternation_mark:',
    ':trident:',
    ':black_small_square:',
    ':white_small_square:',
    ':black_medium_small_square:',
    ':white_medium_small_square:',
    ':black_medium_square:',
    ':white_medium_square:',
    ':black_large_square:',
    ':white_large_square:',
    ':white_check_mark:',
    ':black_square_button:',
    ':white_square_button:',
    ':black_circle:',
    ':white_circle:',
    ':red_circle:',
    ':large_blue_circle:',
    ':large_blue_diamond:',
    ':large_orange_diamond:',
    ':small_blue_diamond:',
    ':small_orange_diamond:',
    ':small_red_triangle:',
    ':small_red_triangle_down:',
    ':shipit:',
)
USER_AGENTS = [
    'Mozilla/5.0 (Linux; Android 6.0.1; SM-G920V Build/MMB29K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.98 Mobile Safari/537.36',
    'Mozilla/5.0 (Linux; Android 5.1.1; SM-G928X Build/LMY47X) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.83 Mobile Safari/537.36',
    'Mozilla/5.0 (Linux; Android 6.0.1; Nexus 6P Build/MMB29P) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.83 Mobile Safari/537.36',
    'Mozilla/5.0 (Linux; Android 6.0.1; E6653 Build/32.2.A.0.253) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.98 Mobile Safari/537.36',
    'Mozilla/5.0 (Linux; Android 6.0; HTC One M9 Build/MRA58K) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.98 Mobile Safari/537.36',
    'Mozilla/5.0 (Linux; Android 6.0.1; SGP771 Build/32.2.A.0.253; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/52.0.2743.98 Safari/537.36',
    'Mozilla/5.0 (Linux; Android 4.4.3; KFTHWI Build/KTU84M) AppleWebKit/537.36 (KHTML, like Gecko) Silk/47.1.79 like Chrome/47.0.2526.80 Safari/537.36',
    'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.135 Safari/537.36 Edge/12.246',
    'Mozilla/5.0 (X11; CrOS x86_64 8172.45.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.64 Safari/537.36',
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_2) AppleWebKit/601.3.9 (KHTML, like Gecko) Version/9.0.2 Safari/601.3.9',
    'Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/47.0.2526.111 Safari/537.36',
    'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0) Gecko/20100101 Firefox/15.0.1',
    'Mozilla/5.0 (Linux; <Android Version>; <Build Tag etc.>) AppleWebKit/<WebKit Rev>(KHTML, like Gecko) Chrome/<Chrome Rev> Safari/<WebKit Rev>',
    'Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19',
    'Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Build/KLP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30',
]

HASHTAGS = {
    'general': [
        '#fun',
        '#instalike',
        '#me',
        '#l4l',
        '#love',
        '#picoftheday',
        '#follow',
        '#instagramanet',
        '#likeforlike',
        '#follow4follow',
        '#happy',
        '#smile',
        '#instatag',
        '#photooftheday',
        '#friends',
        '#followme',
        '#fashion',
        '#instadaily',
        '#summer',
        '#amazing',
        '#like4like',
        '#beautiful',
        '#swag',
        '#igers',
        '#bestoftheday',
        '#tflers',
        '#instagood',
        '#selfie',
        '#cute',
        '#repost',
        '#all_shots',
        '#eyes #pretty',
        '#night',
        '#followforfollow',
        '#lol',
        '#iphoneonly',
        '#instagramanet',
        '#instafollow',
        '#tweegram',
        '#l4l',
        '#my',
        '#party',
        '#style',
        '#fitness',
        '#instatag',
        '#bored',
        '#life',
        '#girls',
        '#cool',
        '#nature',
        '#instamood',
        '#funny',
        '#instacool',
        '#f4f',
        '#instago',
        '#nofilter',
        '#hot',
        '#tbt',
        '#doubletap',
        '#harrystyles',
        '#model',
        '#instagramhub',
        '#home',
        '#instalove',
        '#instatag',
        '#instapic',
        '#picstitch',
        '#awesome',
        '#beauty',
        '#day',
        '#photo',
        '#pink',
        '#tired',
        '#followback',
        '#igdaily',
        '#loveit',
        '#makeup',
        '#instagramanet',
        '#instaphoto',
        '#shoutout',
        '#iphonesia',
        '#20likes',
        '#throwback',
        '#throwbackthursday',
        '#instagramers',
        '#colorful',
    ],
    'girls': [
        '#hot',
        '#girlsnight',
        '#ladiesnight',
        '#follow',
        '#beautiful',
        '#lady',
        '#woman',
        '#instagood',
        '#instalike',
        '#girl',
        '#girlswholikegirls',
        '#followme',
        '#russianwomen',
        '#me',
        '#followback',
        '#ladies',
        '#instagirls',
        '#girlsgeneration',
        '#girlsday',
        '#girlsbestfriend',
        '#sexy',
        '#swag',
        '#instadaily',
        '#womencrushwednesday',
        '#instagramanet',
        '#womancrushwednesday',
        '#love',
        '#girls',
        '#women',
        '#teen',
        '#instatag',
        '#cute',
    ],
    'boys': [
        '#lover',
        '#boyfriend',
        '#instag',
        '#love',
        '#men',
        '#guys',
        '#me',
        '#guy',
        '#me',
        '#dear',
        '#instagramanet',
        '#followforfollow',
        '#photooftheday',
        '#lovers',
        '#hot',
        '#follow',
        '#handsome',
        '#followhim',
        '#boy',
        '#follow4follow',
        '#man',
        '#love',
        '#likes4likes',
        '#dude',
        '#boys',
        '#likesforlikes',
        '#followback',
        '#swag',
    ],
    'love': [
        '#forever',
        '#relationships',
        '#romance',
        '#hug',
        '#relationship',
        '#hugs',
        '#girlfriend',
        '#lovehim',
        '#kiss',
        '#boyfriend',
        '#kisses',
        '#bff',
        '#hearts',
        '#couples',
        '#adorable',
        '#couple',
        '#bf',
        '#russianwomen',
        '@instagramanet',
        '#heart',
        '#girlfriends',
        '#amour',
        '#loveher',
        '#gf',
        '#love',
        '#together',
        '#instalove',
        '#loves',
        '#instagramanet',
    ],
    'friends': [
        '#funny',
        '#friendsforlife',
        '#smile',
        '#bestfriend',
        '#friendsoftheworld',
        '#fun',
        '#party',
        '#goodtime',
        '#myfriends',
        '#instagramanet',
        '#friendsgiving',
        '#bestfriends',
        '#goodtimes',
        '#friends4life',
        '#friendship',
        '#myfriend',
        '#forever',
        '#goodfriends',
        '#awesome',
        '#friendsforever',
        '#friends4ever',
        '#friendships',
        '#memories',
        '#friendshipismagic',
        '#lovethem',
        '#instatag',
        '#friends',
        '#bestfriendsforever',
        '#live',
    ],
    'family': [
        '#familyfirst',
        '#familyphoto',
        '#dad',
        '#instatag',
        '#familyovereverything',
        '#familyforever',
        '#fam',
        '#familytrip',
        '#familydinner',
        '#family1st',
        '#brother',
        '#family',
        '#familyvacation',
        '#familyiseverything',
        '#familytime',
        '#mom',
        '#sister',
        '#familyfun',
        '#familyguy',
        '#bro',
        '#sisters',
        '#familyreunion',
        '#brothers',
        '#familynight',
        '#familyisforever',
        '#familyday',
        '#familylove',
        '#instagramanet',
        '#sis',
    ],
    'nature': [
        '#mothernature',
        '#natureporn',
        '#nature_shooters',
        '#beautifulday',
        '#nature_perfection',
        '#nice',
        '#nature',
        '#sunrise',
        '#day',
        '#sky',
        '#happylife',
        '#instatag',
        '#natureza',
        '#weather',
        '#naturehippys',
        '#naturelover',
        '#sun',
        '#tree',
        '#real',
        '#pretty',
        '#outdoors',
        '#photooftheday',
        '#instagramanet',
        '#blue',
        '#natureaddict',
        '#sunset',
        '#naturelovers',
        '#beautiful',
    ],
    'travel': [
        '#travel',
        '#traveling',
        '#TagsForLikes',
        '#TFLers',
        '#vacation',
        '#visiting',
        '#instatravel',
        '#instago',
        '#instagood',
        '#trip',
        '#holiday',
        '#photooftheday',
        '#fun',
        '#travelling',
        '#tourism',
        '#tourist',
        '#instapassport',
        '#instatraveling',
        '#mytravelgram',
        '#travelgram',
        '#travelingram',
        '#igtravel',
    ],
    'cars': [
        '#cars',
        '#car',
        '#ride',
        '#drive',
        '#TagsForLikes',
        '#driver',
        '#sportscar',
        '#vehicle',
        '#vehicles',
        '#street',
        '#road',
        '#freeway',
        '#highway',
        '#sportscars',
        '#exotic',
        '#exoticcar',
        '#exoticcars',
        '#speed',
        '#tire',
        '#tires',
        '#spoiler',
        '#muffler',
        '#race',
        '#racing',
        '#wheel',
        '#wheels',
        '#rim',
        '#rims',
        '#engine',
        '#horsepower',
    ],
    'sport': [
        '#sports',
        '#sport',
        '#active',
        '#fit',
        '#TagsForLikes',
        '#football',
        '#soccer',
        '#basketball',
        '#futball',
        '#ball',
        '#gametime',
        '#fun',
        '#game',
        '#games',
        '#crowd',
        '#fans',
        '#play',
        '#playing',
        '#player',
        '#field',
        '#green',
        '#grass',
        '#score',
        '#goal',
        '#action',
        '#kick',
        '#throw',
        '#pass',
        '#win',
        '#winning',
    ],
    'tumblr': [
        '#tumblr',
        '#tumblrlife',
        '#tumblrphoto',
        '#TagsForLikes',
        '#tumblrphotos',
        '#tumblrlove',
        '#tumblrpic',
        '#tumblrpics',
        '#tumblrposts',
        '#tumblrpost',
        '#perfect',
        '#tumblrpicture',
        '#tumblrpictures',
        '#tumblrthings',
        '#tumblrstuff',
        '#instatumblr',
        '#beautiful',
        '#love',
        '#TagsForLikes',
        '#tumblrgram',
    ],
}

NETWORK_PROTOCOLS = {
    'application': [
        'ADC',
        'AFP',
        'AMQP',
        'BACnet',
        'BOOTP',
        'BitTorrent',
        'CAMEL',
        'DHCP',
        'DICOM',
        'DICT',
        'DNS',
        'DSM-CC',
        'DSNP',
        'Diameter',
        'ED2K',
        'FTP',
        'Finger',
        'Gnutella',
        'Gopher',
        'HTTP',
        'HTTPS',
        'IMAP',
        'IRC',
        'ISUP',
        'LDAP',
        'MAP',
        'MIME',
        'MQTT',
        'MSNP',
        'Modbus',
        'Mosh',
        'NNTP',
        'NTCIP',
        'NTP',
        'OpenADR',
        'POP3',
        'RADIUS',
        'RDP',
        'RTP',
        'RTSP',
        'Rlogin',
        'SIP',
        'SISNAPI',
        'SMB',
        'SMTP',
        'SNMP',
        'SOAP',
        'SSDP',
        'SSH',
        'STUN',
        'TCAP',
        'TFTP',
        'TUP',
        'Telnet',
        'Universal Plug and Play',
        'WebDAV',
        'rsync',
    ],
    'data_link': [
        'ARCnet',
        'ARP',
        'CDP',
        'DCAP',
        'DMLT',
        'DSMLT',
        'DTP',
        'Econet',
        'Ethernet',
        'FDDI',
        'HDLC',
        'IEEE 802.11',
        'IEEE 802.16',
        'ITU-T G.hn',
        'L2F',
        'L2TP',
        'LACP',
        'LAPD',
        'LLDP',
        'LLDP-MED',
        'LattisNet',
        'LocalTalk',
        'MAC',
        'MLT',
        'PAgP',
        'PPP',
        'Q.710',
        'SLIP',
        'SMLT',
        'STP',
        'StarLAN',
        'Token ring',
        'VLAN',
        'VTP',
    ],
    'network': [
        'CLNP',
        'EGP',
        'EIGRP',
        'GRE',
        'ICMP',
        'IGMP',
        'IGRP',
        'IPSec',
        'IPX',
        'IPv4',
        'IPv6',
        'OSPF',
        'RIP',
        'Routed-SMLT',
        'SCCP',
    ],
    'physical': [
        '10BASE2',
        '10BASE5',
        '10BASE-T',
        '100BASE-FX',
        '100BASE-T',
        '100BASE-TX',
        '1000BASE-SX',
        '1000BASE-T',
        'ARINC 818',
        'Bluetooth',
        'CAN bus',
        'DSL',
        'EIA RS-232',
        'EIA-422',
        'EIA-423',
        'Etherloop',
        'G.hn/G.9960',
        'GSM',
        'IEEE 1394',
        'IRDA',
        'ISDN',
        'MIPI',
        'OTN',
        'RS-449',
        'RS-485',
        'SDH',
        'SONET',
        'TransferJet',
        'USB',
        'Wi-Fi',
    ],
    'presentation': [
        'AFP',
        'TLS',
    ],
    'session': [
        'H.245',
        'L2F',
        'L2TP',
        'NCP',
        'NFS',
        'NetBEUI',
        'NetBIOS',
        'PAP',
        'PPTP',
        'RPC',
        'RTCP',
        'SDP',
        'SIP',
        'SMB',
        'SMPP',
        'SOCKS',
        'SSH',
        'iSNS',
    ],
    'transport': [
        'AH',
        'BGP',
        'DCCP',
        'ESP',
        'FCP',
        'IL',
        'NBF',
        'SCTP',
        'SPX',
        'Sinec H1',
        'TCP',
        'UDP',
        'UDP Lite',
        'iSCSI',
    ],
}



from mimesis.data import MATH_FORMULAS
from mimesis.utils import pull

from .base import BaseProvider


class Science(BaseProvider):
    """Class for getting scientific data"""

    def __init__(self, *args, **kwargs):
        """
        :param locale: Current language.
        """
        super().__init__(*args, **kwargs)
        self._data = pull('science.json', self.locale)

    def math_formula(self):
        """Get a random mathematical formula.

        :return: Math formula.
        :Example:
            A = (ab)/2.
        """
        formula = self.random.choice(MATH_FORMULAS)
        return formula

    def chemical_element(self, name_only=True):
        """Generate a random chemical element.

        :param name_only: If False then will be returned dict.
        :return: Name of chemical element or dict.
        :Example:
            {'Symbol': 'S', 'Name': 'Sulfur', 'Atomic number': '16'}
        """
        elements = self._data['chemical_element']
        nm, sm, an = self.random.choice(elements).split('|')

        if not name_only:
            return {
                'name': nm.strip(),
                'symbol': sm.strip(),
                'atomic_number': an.strip(),
            }

        return nm.strip()

    def scientific_article(self):
        """Generate a random link to scientific article on Wikipedia.

        :return: Link to article on Wikipedia.
        :Example:
            https://en.wikipedia.org/wiki/Black_hole
        """
        articles = self._data['article']
        return self.random.choice(articles)



import sys

from mimesis.data import FOLDERS, PROGRAMMING_LANGS, PROJECT_NAMES
from mimesis.providers import BaseProvider
from mimesis.providers.personal import Personal
from mimesis.constants.platforms import PLATFORMS


class Path(BaseProvider):
    """Class that provides methods and property for generate paths."""
    def __init__(self, platform=sys.platform, *args, **kwargs):
        """
        :param platform: Required platform type ('linux2', 'darwin', 'win32',
        'win64').
        Supported platforms: mimesis/constant/platforms.py
        """
        super().__init__(*args, **kwargs)
        self.__p = Personal('en')
        self.platform = platform

    @property
    def root(self):
        """Generate a root dir path.

        :return: Root dir.
        :Example:
            /
        """
        for platform in PLATFORMS:
            if self.platform == PLATFORMS[platform]['name']:
                root = PLATFORMS[platform]['root']
                return root

    @property
    def home(self):
        """Generate a home path.

        :return: Home path.
        :Example:
            /home/
        """
        for platform in PLATFORMS:
            if self.platform == PLATFORMS[platform]['name']:
                home = PLATFORMS[platform]['home']
                return home

    def user(self, gender='female'):
        """Generate a random user.

        :param gender: Gender of user.
        :return: Path to user.
        :Example:
            /home/oretha
        """
        user = self.__p.name(gender)
        user = user.capitalize() if \
            self.platform == 'win32' else user.lower()
        return self.home + user

    def users_folder(self, user_gender='female'):
        """Generate a random path to user's folders.

        :return: Path.
        :Example:
            /home/taneka/Pictures
        """
        folder = self.random.choice(FOLDERS)
        user = self.user(user_gender)
        for platform in PLATFORMS:
            if self.platform == PLATFORMS[platform]['name']:
                path_separator = PLATFORMS[platform]['path_separator']
                users_folder = (user + '{}' + folder).format(path_separator)
                return users_folder

    def dev_dir(self, user_gender='female'):
        """Generate a random path to development directory.

        :param user_gender: Path to dev directory.
        :return: Path.
        :Example:
            /home/sherrell/Development/Python/mercenary
        """
        dev_folder = self.random.choice(['Development', 'Dev'])
        stack = self.random.choice(PROGRAMMING_LANGS)
        user = self.user(user_gender)
        for platform in PLATFORMS:
            if self.platform == PLATFORMS[platform]['name']:
                path_separator = PLATFORMS[platform]['path_separator']
                dev_dir = (
                    user + '{}' + dev_folder + '{}' + stack
                ).format(path_separator, path_separator)
                return dev_dir

    def project_dir(self, user_gender='female'):
        """Generate a random path to project directory.

        :param user_gender: Gender of user.
        :return: Path to project.
        :Example:
            /home/sherika/Development/Falcon/mercenary
        """
        project = self.random.choice(PROJECT_NAMES)
        for platform in PLATFORMS:
            if self.platform == PLATFORMS[platform]['name']:
                path_separator = PLATFORMS[platform]['path_separator']
                project_dir = (
                    self.dev_dir(user_gender) + '{}' + project
                ).format(path_separator)
                return project_dir



from random import choice, randint

from mimesis.exceptions import JSONKeyError
from mimesis.providers.code import Code
from mimesis.utils import pull

# Internal
_custom_code = Code().custom_code


class RussiaSpecProvider(object):
    """Specific data for russian language (ru)"""

    class Meta:
        name = 'russia_provider'

    @staticmethod
    def generate_sentence():
        """Generate sentence from the parts.

        :return: Sentence.
        :rtype: str
        """
        data = pull('text.json', 'ru')['sentence']
        sentence = [choice(data[k]) for k in ('head', 'p1', 'p2', 'tail')]
        return '{0} {1} {2} {3}'.format(*sentence)

    @staticmethod
    def patronymic(gender='female'):
        """Generate random patronymic name.

        :param gender: Gender of person.
        :return: Patronymic name.
        :Example:
            Алексеевна.
        """
        gender = gender.lower()

        try:
            patronymic = pull('personal.json', 'ru')['patronymic']
            return choice(patronymic[gender])
        except:
            raise JSONKeyError(
                'Not exist key. Please use one of ["female", "male"]')

    @staticmethod
    def passport_series(year=None):
        """Generate random series of passport.

        :param year: Year of manufacture.
        :return: Series.
        :Example:
            02 15.
        """
        year = randint(10, 16) if not \
            year else year

        region = randint(1, 99)
        return '{:02d} {}'.format(region, year)

    @staticmethod
    def passport_number():
        """Generate random passport number.

        :return: Number.
        :Example:
            560430
        """
        return randint(100000, 999999)

    def series_and_number(self):
        """Generate a random passport number and series.

        :return: Series and number.
        :Example:
            57 16 805199.
        """
        return '%s %s' % (
            self.passport_series(),
            self.passport_number(),
        )

    @staticmethod
    def snils():
        """Generate Individual insurance account number (SNILS).
        This method does not generate SNILS using algorithm and
        it's mean that SNILS generated by this method can be invalid.

        :return: SNILS.
        :Example:
            451-952-540-41.
        """
        mask = '###-###-###-##'
        snils = _custom_code(mask=mask)
        return snils



from string import ascii_uppercase

from mimesis.data import IMEI_TACS, ISBN_GROUPS, LOCALE_CODES
from mimesis.providers import BaseProvider
from mimesis.utils import luhn_checksum


class Code(BaseProvider):
    """Class that provides methods for generating codes (isbn, asin & etc.)"""

    def __init__(self, *args, **kwargs):
        """
        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)

    def custom_code(self, mask='@###', char='@', digit='#'):
        """Generate custom code using ascii uppercase and random integers.

        :param mask: Mask of code.
        :param char: Placeholder for characters.
        :param digit: Placeholder for digits.
        :return: Custom code.
        :Example:
            5673-AGFR-SFSFF-1423-4/AD.
        """
        code = ''
        for p in mask:
            if p == char:
                code += self.random.choice(ascii_uppercase)
            elif p == digit:
                code += str(self.random.randint(0, 9))
            else:
                code += p

        return code

    def locale_code(self):
        """Get a random locale code (MS-LCID).
        See Windows Language Code Identifier Reference for more information.

        :return: Locale code.
        :rtype: str
        :Example:
            de-ch
        """
        locale = self.random.choice(LOCALE_CODES)
        return locale

    def issn(self, mask=None):
        """Generate a random International Standard Serial Number (ISSN).

        :param mask: Mask ISSN.
        :return: ISSN.
        """
        if not mask:
            mask = '####-####'
        return self.custom_code(mask=mask)

    def isbn(self, fmt='isbn-10'):
        """Generate ISBN for current locale. Default is ISBN 10,
        but you also can use ISBN-13.

        :param fmt: ISBN format.
        :return: ISBN.
        :Example:
            132-1-15411-375-8.
        """
        groups = ISBN_GROUPS

        mask = '###-{0}-#####-###-#' if \
            fmt == 'isbn-13' else '{0}-#####-###-#'

        if self.locale in groups:
            mask = mask.format(groups[self.locale])
        else:
            mask = mask.format(groups['default'])

        return self.custom_code(mask=mask)

    def ean(self, fmt='ean-13'):
        """Generate EAN (European Article Number) code. Default is
        EAN-13, but you also can use EAN-8.

        :param fmt: Format of EAN.
        :return: EAN.
        :Example:
            3953753179567.
        """
        mask = '########' if fmt == 'ean-8' \
            else '#############'
        return self.custom_code(mask=mask)

    def imei(self):
        """Generate a random IMEI (International Mobile Station Equipment Identity).

        :return: IMEI.
        :Example:
            353918052107063
        """
        num = self.random.choice(IMEI_TACS) + self.custom_code(mask='######')
        return num + luhn_checksum(num)

    def pin(self, mask='####'):
        """Generate a random PIN code.

        :return: PIN code.
        :Example:
            5241.
        """
        return self.custom_code(mask=mask)



from mimesis.data import (DOMAINS, EMOJI, HASHTAGS, HTTP_METHODS,
                          HTTP_STATUS_CODES, NETWORK_PROTOCOLS, SUBREDDITS,
                          SUBREDDITS_NSFW, USER_AGENTS)
from mimesis.exceptions import WrongArgument
from mimesis.providers import BaseProvider, File
from mimesis.providers.personal import Personal


class Internet(BaseProvider):
    """Class for generate the internet data."""

    @staticmethod
    def content_type(mime_type='application'):
        """Get a random HTTP content type.

        :return: Content type.
        :Example:
            Content-Type: application/json
        """
        fmt = File().mime_type(type_t=mime_type)
        return 'Content-Type: {}'.format(fmt)

    def http_status_code(self, code_only=True):
        """Get a random HTTP status.

        :param code_only: Return only http status code.
        :return: HTTP status.
        :Example:
            200 OK
        """
        status = self.random.choice(HTTP_STATUS_CODES)

        if code_only:
            return status.split()[0]
        return status

    def http_method(self):
        """Get a random HTTP method.

        :return: HTTP method.
        :Example:
            POST
        """
        return self.random.choice(HTTP_METHODS)

    def ip_v4(self):
        """Generate a random IPv4 address.

        :return: Random IPv4 address.
        :Example:
            19.121.223.58
        """
        ip = '.'.join([str(self.random.randint(0, 255)) for _ in range(4)])
        return ip

    def ip_v6(self):
        """Generate a random IPv6 address.

        :return: Random IPv6 address.
        :Example:
            2001:c244:cf9d:1fb1:c56d:f52c:8a04:94f3
        """
        ip = '2001:' + ':'.join(
            '%x' % self.random.randint(0, 16 ** 4) for _ in range(7))
        return ip

    def mac_address(self):
        """Generate a random MAC address.

        :return: Random MAC address.
        :Example:
            00:16:3e:25:e7:b1
        """
        mac_hex = [
            0x00, 0x16, 0x3e,
            self.random.randint(0x00, 0x7f),
            self.random.randint(0x00, 0xff),
            self.random.randint(0x00, 0xff),
        ]
        mac = map(lambda x: '%02x' % x, mac_hex)
        return ':'.join(mac)

    def emoji(self):
        """Get a random emoji shortcut code.

        :return: Emoji code.
        :Example:
            :kissing:
        """
        return self.random.choice(EMOJI)

    @staticmethod
    def image_placeholder(width='400', height='300'):
        """Generate a link to the image placeholder.

        :param width: Width of image.
        :param height: Height of image.
        :return: URL to image placeholder.
        """
        url = 'http://placehold.it/%sx%s'
        return url % (width, height)

    def stock_image(self, category=None, width=1900, height=1080):
        """Get a random beautiful stock image that hosted on Unsplash.com

        :param category: Category of image. Available: 'buildings', 'food',
        'nature', 'people', 'technology', 'objects'.
        :param width: Width of the image.
        :param height: Height of the image.
        :return: An image (Link to image).
        """
        url = 'https://source.unsplash.com/category/' \
              '{category}/{width}x{height}'

        categories = (
            'buildings', 'food', 'nature',
            'people', 'technology', 'objects',
        )

        if not category or category not in categories:
            category = self.random.choice(categories)

        return url.format(category=category, width=width, height=height)

    def image_by_keyword(self, keyword=None):
        url = 'https://source.unsplash.com/weekly?{keyword}'

        keywords = [
            'cat', 'girl', 'boy', 'beauty',
            'nature', 'woman', 'man', 'tech',
            'space',
        ]

        if not keyword:
            keyword = self.random.choice(keywords)

        return url.format(keyword=keyword)

    def hashtags(self, quantity=4, category='general'):
        """Create a list of hashtags (for Instagram, Twitter etc.)

        :param quantity: The quantity of hashtags.
        :type quantity: int
        :param category: Available categories: general, girls, love,
        boys, friends, family, nature, travel, cars, sport, tumblr.
        :return: The list of hashtags.
        :rtype: list

        :Example:
            ['#love', '#sky', '#nice'].
        """
        category = category.lower()
        supported = ''.join(list(HASHTAGS.keys()))

        try:
            hashtags = HASHTAGS[category]
        except KeyError:
            raise KeyError('Unsupported category. Use: {}'.format(supported))

        if int(quantity) == 1:
            return self.random.choice(hashtags)

        tags = [self.random.choice(hashtags) for _ in range(int(quantity))]
        return tags

    def home_page(self, gender='female'):
        """Generate a random home page.

        :param gender: Gender of author of site.
        :return: Random home page.
        :Example:
            http://www.font6.info
        """
        url = 'http://www.' + Personal().username(gender)
        domain = self.random.choice(DOMAINS)
        return '{}{}'.format(url, domain)

    def subreddit(self, nsfw=False, full_url=False):
        """Get a random subreddit from the list.

        :param nsfw: NSFW subreddit.
        :param full_url: Full URL address.
        :return: Subreddit or URL to subreddit.
        :Example:
            https://www.reddit.com/r/flask/
        """
        url = 'http://www.reddit.com'
        if not nsfw:
            if not full_url:
                return self.random.choice(SUBREDDITS)
            else:
                return url + self.random.choice(SUBREDDITS)

        nsfw_sr = self.random.choice(SUBREDDITS_NSFW)
        result = url + nsfw_sr if full_url else nsfw_sr
        return result

    def user_agent(self):
        """Get a random user agent.

        :return: User agent.
        :Example:
            Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:15.0)
            Gecko/20100101 Firefox/15.0.1
        """
        return self.random.choice(USER_AGENTS)

    def network_protocol(self, layer='application'):
        """Get a random network protocol form OSI model.

        :param layer: Layer of protocol: application, data_link,
        network, physical, presentation, session and transport.
        :return: Protocol name.
        :Example:
            AMQP
        """
        # TODO: Refactoring.
        layer = layer.lower()
        try:
            protocol = self.random.choice(NETWORK_PROTOCOLS[layer])
            return protocol
        except KeyError:
            protocols = list(NETWORK_PROTOCOLS.keys())
            raise WrongArgument('Unsupported layer, use: {}'.format(protocols))



# utils.py

import collections
import functools
import json
import os.path as path
import urllib.request as request

from mimesis.exceptions import UnsupportedLocale
from mimesis.settings import SUPPORTED_LOCALES

__all__ = ['pull', 'download_image', 'locale_info']

PATH = path.abspath(path.join(path.dirname(__file__), 'data'))


def locale_info(locale: str) -> str:
    """Return name (in english) or local name of the locale

    :param locale: Locale abbreviation.
    :type locale: str
    :returns: Locale name.
    :rtype: str
    :Example:

    >>> from mimesis.utils import locale_info
    >>> locale_info('sv')
    'Swedish'
    """
    locale = locale.lower()

    if locale not in SUPPORTED_LOCALES:
        raise UnsupportedLocale('Locale %s is not supported' % locale)

    return SUPPORTED_LOCALES[locale]['name']


def luhn_checksum(num) -> str:
    """Calculate a checksum for num using the Luhn algorithm.

    See: https://en.wikipedia.org/wiki/Luhn_algorithm
    :param num: The number to calculate a checksum for as a string
    :type num: str
    :returns: checksum for number
    :rtype: str
    :Example:

    >>> from mimesis.utils import luhn_checksum
    >>> luhn_checksum("7992739871")
    '3'
    """
    check = 0
    for i, s in enumerate(reversed([x for x in num])):
        sx = int(s)
        sx = sx * 2 if i % 2 == 0 else sx
        sx = sx - 9 if sx > 9 else sx
        check += sx
    return str(check * 9 % 10)


@functools.lru_cache(maxsize=None)
def pull(file, locale='en') -> dict:
    """Open json file file and get content from file and memorize result using
     lru_cache.

    .. note:: pull - is internal function, please do not use this function
    outside the module 'mimesis'.

    :param file: The name of file.
    :param locale: Locale.
    :returns: The content of the file.

    :Example:

        >>> from mimesis.utils import pull
        >>> en = pull(file='datetime.json', locale='en')
        >>> isinstance(en, dict)
        True
        >>> en['day']['abbr'][0]
        'Mon.'
    """

    def get_data(locale_name):
        """
        Pull JSON data from file.
        :param locale_name: Name of locale to pull.
        :return: Dict of data from file
        """
        file_path = path.join(PATH + '/' + locale_name, file)
        # Needs explicit encoding for Windows
        with open(file_path, 'r', encoding='utf8') as f:
            return json.load(f)

    locale = locale.lower()

    if locale not in SUPPORTED_LOCALES:
        raise UnsupportedLocale('Locale %s is not supported' % locale)

    master_locale = locale.split('-')[0]
    data = get_data(master_locale)

    # Handle sub-locales
    if '-' in locale:
        data = update_dict(data, get_data(locale))

    return data


def download_image(url, save_path='', unverified_ctx=False):
    """Download image and save in current directory on local machine.

    :param url: URL to image.
    :param save_path: Saving path.
    :param unverified_ctx: Create unverified context. Use if you get
    CERTIFICATE_VERIFY_FAILED.
    :return: Image name.
    :rtype: str
    :Example:
        f88684c22086d4bb3983159fb1e95c22.png
    """
    if unverified_ctx:
        import ssl
        try:
            ssl._create_default_https_context = ssl._create_stdlib_context
        except AttributeError:
            raise NotImplementedError(
                'unverified_ctx is only supported in Python 3.4+',
            )

    if url is not None:
        image_name = url.rsplit('/')[-1]
        request.urlretrieve(url, save_path + image_name)
        return image_name
    return None


def update_dict(initial, other):
    """Recursively update a dictionary.

    :param initial: Dict to update.
    :param other: Dict to update from.
    :return: Updated dict.
    :rtype: dict
    """
    for key, value in other.items():
        if isinstance(value, collections.Mapping):
            r = update_dict(initial.get(key, {}), value)
            initial[key] = r
        else:
            initial[key] = other[key]
    return initial



from distutils.core import setup

from mimesis import __author__, __version__

with open('PYPI_README.rst', 'r', encoding='utf-8') as f:
    readme = f.read()

setup(
    name='mimesis',
    version=__version__,
    packages=[
        'mimesis',
        'mimesis.data',
        'mimesis.data.int',
        'mimesis.builtins',
        'mimesis.constants',
        'mimesis.providers',
    ],
    keywords=[
        'fake',
        'mock',
        'data',
        'populate',
        'database',
        'testing',
        'generate',
        'mimesis',
        'dummy',
    ],
    package_data={
        'mimesis': [
            'data/*/*',
        ],
    },
    data_files=[
        ('', ['LICENSE',
              'PYPI_README.rst',
              ],
         ),
    ],
    url='https://github.com/lk-geimfari/mimesis',
    license='MIT License',
    author=__author__,
    author_email='likid.geimfari@gmail.com',
    description='Mimesis: dummy data for developers.',
    long_description=readme,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development',
        'Topic :: Software Development :: Testing',
    ],
)



import json

from mimesis.decorators import type_to
from mimesis.exceptions import UndefinedSchema
from mimesis.providers import Generic


class Schema(object):
    def __init__(self, locale=None):
        self.schema = {}
        if locale is None:
            self.locale = 'en'
        else:
            self.locale = locale

        self.generic = Generic(self.locale)

    def __generate(self, schema):
        data = dict()
        for k, v in schema.items():
            if isinstance(v, dict):
                data[k] = self.__generate(v)
            elif isinstance(v, list):
                data[k] = [self.__generate(i) for i in v]
            else:
                provider, method = v.split('.')
                data[k] = getattr(
                    getattr(self.generic, provider), method)()
        return data

    def load(self, path=None, schema=None):

        if schema:
            self.schema = schema
        if path:
            try:
                try:
                    with open(path, 'r') as f:
                        self.schema = json.load(f)
                except FileNotFoundError:
                    raise FileNotFoundError(
                        'File {path} is not found'.format(path=path))
            except ValueError:
                raise ValueError('Invalid json file!')
        return self

    @type_to(list, check_len=True)
    def create(self, iterations=1):
        if self.schema:
            return map(lambda _: self.__generate(self.schema),
                       range(iterations))
        else:
            raise UndefinedSchema(
                'The schema is empty or do not loaded.')



from mimesis.data import GAMES, GAMING_PLATFORMS, GENRES, SCORE_PHRASES
from mimesis.providers import BaseProvider

__all__ = ['Games']


class Games(BaseProvider):
    def gaming_platform(self):
        """Get random gaming platform.

        :return: Gaming platform
        :Example:
            PlayStation 4 Pro
        """
        return self.random.choice(GAMING_PLATFORMS)

    def score(self, minimum=1, maximum=10):
        return self.random.randint(minimum * 10, maximum * 10) / 10

    def pegi_rating(self, pt=False):
        """Get a random PEGI rating.

        :param pt: PEGI rating for Portugal.
        :return: PEGI rating.
        :Example:
            PEGI 18
        """
        standard = [3, 7, 12, 16, 18]

        # In Portugal, two of the PEGI categories were aligned with the age
        # ratings of the film classification system to avoid confusion;
        # 3 was changed to 4 and 7 was changed to 6.
        if pt:
            standard[0] = 4
            standard[1] = 6

        return '{0} {1}'.format('PEGI',
                                self.random.choice(standard))

    def genre(self):
        """Get a random genre of game.

        :return: Genre.
        :Example:
            Shooter
        """
        return self.random.choice(GENRES)

    def score_phrase(self):
        """Get a random score phrase.

        :return: Score phrase.
        :Example:
            Great
        """
        return self.random.choice(SCORE_PHRASES)

    def game(self):
        """Get a random game from list of games.

        :return: Name of the game.
        :Example:
            Battlefield 1
        """
        return self.random.choice(GAMES)



import re
from string import ascii_letters, digits, punctuation

from mimesis.data import (BLOOD_GROUPS, EMAIL_DOMAINS, ENGLISH_LEVEL,
                          FAVORITE_MUSIC_GENRE, GENDER_SYMBOLS,
                          SEXUALITY_SYMBOLS)
from mimesis.exceptions import WrongArgument
from mimesis.providers import BaseProvider, Code
from mimesis.providers.cryptographic import Cryptographic
from mimesis.utils import luhn_checksum, pull

__all__ = ['Personal']


class Personal(BaseProvider):
    """Class for generate personal data, i.e names, surnames,
    age and another."""

    def __init__(self, *args, **kwargs):
        """
        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self.data = pull('personal.json', self.locale)
        self._store = {
            'age': 0,
        }

    def age(self, minimum=16, maximum=66):
        """Get a random integer value.

        :param maximum: max age
        :param minimum: min age
        :return: Random integer (from minimum=16 to maximum=66)
        :Example:
            23.
        """
        a = self.random.randint(int(minimum), int(maximum))
        self._store['age'] = a
        return a

    def child_count(self, max_childs=5):
        """Get a count of child's.

        :param max_childs: Maximum count of child's.
        :return: Ints. Depend on previous generated age.
        """
        a = self._store['age']
        if a == 0:
            a = self.age()

        cc = 0 if a < 18 else self.random.randint(0, max_childs)
        return cc

    def work_experience(self, working_start_age=22):
        """Get a work experience.

        :param working_start_age: Age then person start to work.
        :return: Int. Depend on previous generated age.
        """
        a = self._store['age']
        if a == 0:
            a = self.age()

        return max(a - working_start_age, 0)

    def name(self, gender='female'):
        """Get a random name.

        :param gender: if 'male' then will getting male name else female name.
        :return: Name.
        :Example:
            John.
        """
        # TODO: Add function for checking gender.
        try:
            names = self.data['names'][gender]
        except KeyError:
            raise WrongArgument('gender must be "female" or "male"')
        return self.random.choice(names)

    def surname(self, gender='female'):
        """Get a random surname.

        :param gender: The gender of person.
        :return: Surname.
        :Example:
            Smith.
        """
        # Separated by gender.
        if self.locale in ('ru', 'is', 'uk'):
            try:
                return self.random.choice(self.data['surnames'][gender])
            except KeyError:
                raise WrongArgument('gender must be "female" or "male"')

        surname = self.random.choice(self.data['surnames'])
        return surname

    def title(self, gender='female', title_type='typical'):
        """Get a random title (prefix/suffix) for name.

        :param gender: The gender.
        :param title_type:  The type of title ('typical' and 'academic').
        :return: The title.
        :Example:
            PhD.
        """
        try:
            titles = self.data['title'][gender][title_type]
        except KeyError:
            raise WrongArgument('Wrong value of argument.')

        title = self.random.choice(titles)
        return title

    def full_name(self, gender='female', reverse=False):
        """Generate a random full name.

        :param reverse: if true: surname/name else name/surname
        :param gender: if gender='male' then will be returned male name else
            female name.
        :return: Full name.
        :Example:
            Johann Wolfgang.
        """
        gender = gender.lower()

        fmt = '{1} {0}' if reverse else '{0} {1}'
        return fmt.format(
            self.name(gender),
            self.surname(gender),
        )

    def username(self, gender='female'):
        """Get a random username with digits. Username generated
        from names (en) for all locales.

        :return: Username.
        :rtype: str
        :Example:
            abby1189.
        """
        try:
            names = pull('personal.json', 'en')['names'][gender]
            name = self.random.choice(names)
        except KeyError:
            raise WrongArgument('gender must be "female" or "male"')

        fmt = ['%s_%s', '%s%s', '%s-%s']

        username = self.random.choice(fmt) % (name,
                                              self.random.randint(2, 9999))
        return username.lower()

    def password(self, length=8, algorithm=None):
        """Generate a password or hash of password.

        :param length: Length of password.
        :param algorithm: Hashing algorithm.
        :return: Password or hash of password.
        :Example:
            k6dv2odff9#4h (without hashing).
        """
        password = ''.join([self.random.choice(
            ascii_letters + digits + punctuation) for _ in range(int(length))])

        if algorithm is not None:
            return Cryptographic().hash(algorithm=algorithm)

        return password

    def email(self, gender='female', domains=None):
        """Generate a random email.

        :param gender: Gender of the user.
        :type gender: str
        :param domains: Custom domain for email.
        :type domains: list, tuple
        :return: Email address.
        :Example:
            foretime10@live.com
        """
        host = domains if domains else EMAIL_DOMAINS

        email = self.username(gender) + self.random.choice(host)
        return email

    def bitcoin(self):
        """Generate a random bitcoin address. Currently supported only two
        address formats that are most popular: 'P2PKH' and 'P2SH'

        :return: Bitcoin address.
        :Example:
            3EktnHQD7RiAE6uzMj2ZifT9YgRrkSgzQX
        """
        fmt = self.random.choice(['1', '3'])
        fmt += ''.join([self.random.choice(ascii_letters + digits)
                        for _ in range(33)])
        return fmt

    def cvv(self):
        """Generate a random card verification value (CVV).

        :return: CVV code.
        :rtype: int
        :Example:
            324
        """
        return self.random.randint(100, 999)

    def credit_card_number(self, card_type='visa'):
        """Generate a random credit card number.

        :param card_type: Issuing Network. Default is Visa.
        :return: Credit card number.
        :Example:
            4455 5299 1152 2450
        """
        length = 16
        regex = re.compile('(\d{4})(\d{4})(\d{4})(\d{4})')

        if card_type in ('visa', 'vi', 'v'):
            number = self.random.randint(4000, 4999)
        elif card_type in ('master_card', 'mc', 'master', 'm'):
            number = self.random.choice([self.random.randint(2221, 2720),
                                         self.random.randint(5100, 5500)])
        elif card_type in ('american_express', 'amex', 'ax', 'a'):
            number = self.random.choice([34, 37])
            length = 15
            regex = re.compile('(\d{4})(\d{6})(\d{5})')
        else:
            raise NotImplementedError(
                'Card type {} is not supported.'.format(card_type))

        number = str(number)
        while len(number) < length - 1:
            number += self.random.choice(digits)

        card = ' '.join(regex.search(number + luhn_checksum(number)).groups())
        return card

    def credit_card_expiration_date(self, minimum=16, maximum=25):
        """Generate a random expiration date for credit card.

        :param minimum: Date of issue.
        :param maximum: Maximum of expiration_date.
        :return: Expiration date of credit card.
        :rtype: str
        :Example:
            03/19.
        """
        month, year = [self.random.randint(1, 12),
                       self.random.randint(minimum, maximum)]
        month = '0' + str(month) if month < 10 else month
        return '{0}/{1}'.format(month, year)

    def cid(self):
        """Generate a random CID code.

        :return: CID code.
        :Example:
            7452
        """
        return self.random.randint(1000, 9999)

    def paypal(self):
        """Generate a random PayPal account.

        :return: Email of PapPal user.
        :Example:
            wolf235@gmail.com
        """
        return self.email()

    def social_media_profile(self, gender='female'):
        """Generate profile for random social network.

        :param gender: Gender of user.
        :return: Profile in some network.
        :Example:
            http://facebook.com/some_user
        """
        urls = [
            'facebook.com/{}',
            'twitter.com/{}',
            'medium.com/@{}',
        ]
        url = 'http://' + self.random.choice(urls)
        username = self.username(gender)

        return url.format(username)

    def gender(self, iso5218=False, symbol=False):
        """Get a random title of gender, code for the representation
        of human sexes is an international standard that defines a
        representation of human sexes through a language-neutral single-digit
        code or symbol of gender.

        :param iso5218: Codes for the representation of human sexes
        is an international standard.
        :param symbol: Symbol of gender.
        :return: Title of gender.
        :rtype: str
        :Example:
            Male
        """
        # The four codes specified in ISO/IEC 5218 are:
        #     0 = not known,
        #     1 = male,
        #     2 = female,
        #     9 = not applicable.
        codes = [0, 1, 2, 9]

        if iso5218:
            return self.random.choice(codes)

        if symbol:
            return self.random.choice(GENDER_SYMBOLS)

        gender = self.random.choice(self.data['gender'])
        return gender

    def height(self, minimum=1.5, maximum=2.0):
        """Generate a random height in M (Meter).

        :param minimum: Minimum value.
        :param maximum: Maximum value.
        :return: Height.
        :Example:
            1.85.
        """
        h = self.random.uniform(float(minimum), float(maximum))
        return '{:0.2f}'.format(h)

    def weight(self, minimum=38, maximum=90):
        """Generate a random weight in Kg.

        :param minimum: min value
        :param maximum: max value
        :return: Weight.
        :Example:
            48.
        """
        weight = self.random.randint(int(minimum), int(maximum))
        return weight

    def blood_type(self):
        """Get a random blood type.

        :return: Blood type (blood group).
        :Example:
            A+
        """
        return self.random.choice(BLOOD_GROUPS)

    def sexual_orientation(self, symbol=False):
        """Get a random (LOL) sexual orientation.

        :param symbol: Unicode symbol.
        :return: Sexual orientation.
        :Example:
            Heterosexuality.
        """
        if symbol:
            return self.random.choice(SEXUALITY_SYMBOLS)

        sexuality = self.data['sexuality']
        return self.random.choice(sexuality)

    def occupation(self):
        """Get a random job.

        :return: The name of job.
        :Example:
            Programmer.
        """
        jobs = self.data['occupation']
        return self.random.choice(jobs)

    def political_views(self):
        """Get a random political views.

        :return: Political views.
        :Example:
            Liberal.
        """
        views = self.data['political_views']
        return self.random.choice(views)

    def worldview(self):
        """Get a random worldview.

        :return: Worldview.
        :Example:
            Pantheism.
        """
        views = self.data['worldview']
        return self.random.choice(views)

    def views_on(self):
        """
        Get a random views on.

        :return: Views on.
        :Example:
            Negative.
        """
        views = self.data['views_on']
        return self.random.choice(views)

    def nationality(self, gender='female'):
        """Get a random nationality.

        :param gender: female or male
        :return: Nationality.
        :Example:
            Russian.
        """
        # Subtleties of the Russian and Ukrainian orthography.
        separated_locales = ['ru', 'uk']

        if self.locale in separated_locales:
            nations = self.data['nationality'][gender]
            return self.random.choice(nations)

        return self.random.choice(self.data['nationality'])

    def university(self):
        """
        Get a random university.

        :return: University name.
        :Example:
            MIT.
        """
        universities = self.data['university']
        return self.random.choice(universities)

    def academic_degree(self):
        """Get a random academic degree.

        :return: Degree.
        :Example:
            Bachelor.
        """
        degrees = self.data['academic_degree']
        return self.random.choice(degrees)

    def language(self):
        """Get a random language.

        :return: Random language.
        :Example:
            Irish.
        """
        languages = self.data['language']
        return self.random.choice(languages)

    def favorite_movie(self):
        """Get a random movie for current locale.

        :return: The name of the movie.
        :Example:
            Interstellar.
        """
        movies = self.data['favorite_movie']
        return self.random.choice(movies)

    def favorite_music_genre(self):
        """Get a random music genre.

        :return: A music genre.
        :Example:
            Ambient.
        """
        return self.random.choice(FAVORITE_MUSIC_GENRE)

    def telephone(self, mask=None, placeholder='#'):
        """Generate a random phone number.

        :param mask: Mask for formatting number.
        :param placeholder: A placeholder for a mask (default is #).
        :return: Phone number.
        :Example:
            +7-(963)-409-11-22.
        """
        # Default
        default = '+#-(###)-###-####'

        if not mask:
            masks = self.data.get('telephone_fmt', default)
            mask = self.random.choice(masks)

        return Code(self.locale).custom_code(mask=mask, digit=placeholder)

    def avatar(self, size=256):
        """Generate a random avatar (link to avatar) using API of  Adorable.io.

        :return: Link to avatar.
        :Example:
            https://api.adorable.io/avatars/64/875ed3de1604812b3c2b592c05863f47.png
        """
        url = 'https://api.adorable.io/avatars/{0}/{1}.png'
        return url.format(size, self.password(algorithm='md5'))

    def identifier(self, mask='##-##/##'):
        """Generate a random identifier by mask. With this method you can generate
        any identifiers that you need. Simply select the mask that you need.

        :param mask: The mask. Here '@' is a placeholder for characters
        and '#' is placeholder for digits.
        :return: An identifier.
        :Example:
            07-97/04
        """
        return Code(self.locale).custom_code(mask=mask)

    def level_of_english(self):
        """Get a random level of English.

        :return: Level of english.
        :Example:
            Intermediate.
        """
        return self.random.choice(ENGLISH_LEVEL)



from random import randint


class BrazilSpecProvider(object):
    """Class that provides special data for pt-br"""

    class Meta:
        name = 'brazil_provider'

    @staticmethod
    def cpf(with_mask=True):
        """Get a random CPF (brazilian social security code)

        :param with_mask: use CPF mask (###.###.###-##) in the return
        :returns: Random CPF
        :Example:
            001.137.297-40
        """

        def get_verifying_digit_cpf(cpf, peso):
            """Calculates the verifying digit for the CPF

            :param cpf: ist of integers with the CPF
            :param peso: Integer with the weight for the modulo 11 calculate
            :returns: the verifying digit for the CPF
            """
            soma = 0
            for index, digit in enumerate(cpf):
                soma += digit * (peso - index)
            resto = soma % 11
            if resto == 0 or resto == 1 or resto >= 11:
                return 0
            return 11 - resto

        cpf_without_dv = [randint(0, 9) for _ in range(9)]
        first_dv = get_verifying_digit_cpf(cpf_without_dv, 10)

        cpf_without_dv.append(first_dv)
        second_dv = get_verifying_digit_cpf(cpf_without_dv, 11)
        cpf_without_dv.append(second_dv)

        cpf = ''.join([str(i) for i in cpf_without_dv])

        if with_mask:
            return cpf[:3] + '.' + cpf[3:6] + '.' + cpf[6:9] + '-' + cpf[9:]
        return cpf

    @staticmethod
    def cnpj(with_mask=True):
        """Get a random cnpj (brazilian social security code)

        :param with_mask: use cnpj mask (###.###.###-##) in the return
        :returns: Random cnpj
        :Example:
            77.732.230/0001-70
        """

        def get_verifying_digit_cnpj(cnpj, peso):
            """Calculates the verifying digit for the cnpj
            :param cnpj: list of integers with the cnpj
            :param peso: integer with the weigth for the modulo 11 calcule
            :returns: the verifying digit for the cnpj
            """
            soma = 0
            if peso == 5:
                peso_list = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
            elif peso == 6:
                peso_list = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2]
            for i, _ in enumerate(cnpj):
                soma += peso_list[i] * cnpj[i]
            resto = soma % 11
            if resto < 2:
                return 0
            return 11 - resto

        cnpj_without_dv = [randint(0, 9) for _ in range(12)]

        first_dv = get_verifying_digit_cnpj(cnpj_without_dv, 5)
        cnpj_without_dv.append(first_dv)

        second_dv = get_verifying_digit_cnpj(cnpj_without_dv, 6)
        cnpj_without_dv.append(second_dv)

        cnpj = ''.join([str(i) for i in cnpj_without_dv])

        if with_mask:
            return (
                cnpj[:2] + '.' + cnpj[2:5] + '.' + cnpj[5:8] + '/' +
                cnpj[8:12] + '-' + cnpj[12:]
            )
        return cnpj



from mimesis.data import (BACKEND, CONTAINER, FRONTEND, LICENSES, NOSQL, OS,
                          PROGRAMMING_LANGS, SQL)
from mimesis.providers import BaseProvider


class Development(BaseProvider):
    """Class for getting fake data for Developers."""

    def software_license(self):
        """Get a random software license from list.

        :return: License name.
        :rtype: str
        :Example:
            The BSD 3-Clause License.
        """
        return self.random.choice(LICENSES)

    def version(self):
        """Generate a random version information.

        :return: The version of software.
        :Example:
            0.11.3.
        """
        n = (self.random.randint(0, 11) for _ in range(3))
        return '{}.{}.{}'.format(*n)

    def database(self, nosql=False):
        """Get a random database name.

        :param nosql: only NoSQL databases.
        :return: Database name.
        :Example:
            PostgreSQL.
        """
        if nosql:
            return self.random.choice(NOSQL)
        return self.random.choice(SQL)

    def container(self):
        """Get a random containerization system.

        :return: Containerization system.
        :Example:
            Docker.
        """
        return self.random.choice(CONTAINER)

    def version_control_system(self):
        """Get a random version control system.

        :return: Version control system
        :Example:
            Git
        """
        return self.random.choice(['Git', 'Subversion'])

    def programming_language(self):
        """Get a random programming language from the list.

        :return: Programming language.
        :Example:
            Erlang.
        """
        return self.random.choice(PROGRAMMING_LANGS)

    def backend(self):
        """Get a random backend stack.

        :return: Stack.
        :Example:
            Elixir/Phoenix
        """
        return self.random.choice(BACKEND)

    def frontend(self):
        """Get a random front-end stack.

        :return: Stack.
        :Example:
            JS/React.
        """
        return self.random.choice(FRONTEND)

    def os(self):
        """Get a random operating system or distributive name.

        :return: The name of OS.
        :Example:
            Gentoo
        """
        return self.random.choice(OS)

    def stackoverflow_question(self):
        """Generate a random question id for StackOverFlow
        and return url to a question.

        :return: URL to a question.
        :Example:
            http://stackoverflow.com/questions/1726403
        """
        post_id = self.random.randint(1000000, 9999999)
        url = 'http://stackoverflow.com/questions/{0}'
        return url.format(post_id)



from functools import wraps

from mimesis.data import ROMANIZATION_ALPHABETS
from mimesis.exceptions import UnsupportedLocale


def romanized(locale):
    def romanized_deco(func):
        """Cyrillic letter to latin converter. Romanization of the Cyrillic
         alphabet is the process of transliterating the Cyrillic language from
         the Cyrillic script into the Latin alphabet.

        .. note:: At this moment it's work only for Russian and Ukrainian,
        but in future we can add support for all slavic languages or for all
        Cyrillic languages.

        :param func: Function.
        :return: Latinized text.
        """

        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                alphabet = ROMANIZATION_ALPHABETS[locale]
            except KeyError:
                raise UnsupportedLocale(
                    'Locale {0} is not supported yet.'.format(locale),
                )
            result = func(*args, **kwargs)
            txt = ''.join([alphabet[i] for i in result if i in alphabet])
            return txt

        return wrapper

    return romanized_deco


def type_to(new_type, check_len=False):
    """Convert result of function to different type

    :param new_type: New type.
    :param check_len: Check lenght of object.
    :return: Converted to new_type object.
    """

    def inner(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            result = func(*args, **kwargs)
            result = new_type(result)

            if check_len and len(result) == 1:
                return result[0]

            return result

        return wrapper

    return inner
